@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2002 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: cluster.tcl
-#  Last change:  24 February 2002
+#  Last change:  20 October 2004
 #
 
 proc DistCluster {w dtxt dist args} {
@@ -36,14 +36,14 @@ proc DistCluster {w dtxt dist args} {
 	void -
 	error { return }
     }
-    if { "$args" != "" } {
+    if { $args != "" } {
 	set min [lindex $args 0]
     } else { set min 0 }
     set wgs ""
     foreach ix $wpixs {
 	if { $ix != -1 } { lappend wgs $ix }
     }
-    if { "$wgs" == "" } {
+    if { $wgs == "" } {
 	GMMessage $MESS(allundef)
 	return
     }
@@ -97,16 +97,9 @@ proc DistCluster {w dtxt dist args} {
     set n 0 ; set k 0
     foreach bd $bds {
 	foreach wpix $wgs cl [set cls$k] {
-	    if { "$cl" != "" } {
-		set grname [format "$TXT(cluster) %d" $n]
-		while { [IndexNamed GR $grname] != -1 } {
-		    incr n ; set grname [format "$TXT(cluster) %d" $n]
-		}
-		set data [FormData GR "Name Obs Conts" \
-		      [list $grname \
-		       "$TXT(centre): $WPName($wpix)\n$dtxt" \
-		       [list [list WP $cl]]]]
-		CreateItem GR $data
+	    if { $cl != "" } {
+		CreateGRFor cluster "$TXT(centre): $WPName($wpix)\n$dtxt" \
+		    [list [list WP $cl]]
 	    }
 	}
 	set min $bd
@@ -131,7 +124,7 @@ proc QuadrCluster {w dlat dlong} {
     foreach ix $wpixs {
 	if { $ix != -1 } { lappend wgs $ix }
     }
-    if { "$wgs" == "" } {
+    if { $wgs == "" } {
 	GMMessage $MESS(allundef)
 	return
     }
@@ -139,9 +132,8 @@ proc QuadrCluster {w dlat dlong} {
     foreach cix $wgs {
 	set cl($cix) ""
 	set p $WPPosn($cix)
-	if { "$WPDatum($cix)" != "$datum" } {
-	    set p [ConvertDatum [lindex $p 0] [lindex $p 1] \
-		    $WPDatum($cix) $datum DDD]
+	if { $WPDatum($cix) != $datum } {
+	    set p [ToDatum [lindex $p 0] [lindex $p 1] $WPDatum($cix) $datum]
 	}
 	set gd [AddToGData $gd [lindex $p 0] [lindex $p 1] $cix]
     }
@@ -155,9 +147,8 @@ proc QuadrCluster {w dlat dlong} {
 	    if { $Cluster(abort) } { return }
 	}	    
 	set p $WPPosn($ix)
-	if { "$WPDatum($ix)" != "$datum" } {
-	    set p [ConvertDatum [lindex $p 0] [lindex $p 1] \
-		    $WPDatum($ix) $datum DDD]
+	if { $WPDatum($ix) != $datum } {
+	    set p [ToDatum [lindex $p 0] [lindex $p 1] $WPDatum($ix) $datum]
 	}
 	foreach cix [LookupQuadrGData $gd [lindex $p 0] [lindex $p 1] \
 		$dlat $dlong] {
@@ -171,16 +162,10 @@ proc QuadrCluster {w dlat dlong} {
     set dlongext [ExtDegrees DMSsimpl $dlong]
     set n 0
     foreach cix $wgs {
-	if { "$cl($cix)" != "" } {
-	    set grname [format "$TXT(cluster) %d" $n]
-	    while { [IndexNamed GR $grname] != -1 } {
-		incr n ; set grname [format "$TXT(cluster) %d" $n]
-	    }
-	    set data [FormData GR "Name Obs Conts" \
-		    [list $grname \
-		    "$TXT(centre): $WPName($cix)\n${dlatext}x$dlongext" \
-		    [list [list WP $cl($cix)]]]]
-	    CreateItem GR $data
+	if { $cl($cix) != "" } {
+	    CreateGRFor cluster \
+		"$TXT(centre): $WPName($cix)\n${dlatext}x$dlongext" \
+		[list [list WP $cl($cix)]]
 	}
     }
     return
@@ -307,12 +292,12 @@ proc DoMakeClusters {w} {
     switch $Cluster(how) {
 	bydist {
 	    set min $Cluster(min)
-	    if { "$min" == "" } {
+	    if { $min == "" } {
 		set min 0
 	    } elseif { ! [CheckFloat GMMessage $min] || \
 		    ! [CheckFloat GMMessage $Cluster(max)] } { return }
 	    regexp {(.*)=(.*)} $Cluster(unit) z type unit
-	    if { "$type" == "subdist" } {
+	    if { $type == "subdist" } {
 		set sc [expr $DSCALEFOR(sub,$unit)/$DSCALEFOR($unit)]
 	    } else {
 		set sc [expr 1.0/$DSCALEFOR($unit)]
