@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: command.tcl
-#  Last change:  5 April 2006
+#  Last change:  14 May 2007
 #
 
 # this file is only source-ed in command-line mode (even if the graphical
@@ -138,20 +138,88 @@ proc ExecCommand {} {
 	    CmdGenerateGeoRef
 	    return "-0"
 	}
+	geopicts {
+	    global Number Datum SYMBOLS DEFAULTSYMBOL
+	    set type $COMMAND(intypes)
+	    if { [CmdReadFails $COMMAND(infmt) $COMMAND(inpath) \
+		      $COMMAND(in,params) $type] || $Number($type) == 0 } {
+		return 1
+	    }
+	    set fmt $COMMAND(format)
+	    set ix [expr $Number($type)-1]
+	    set mdate pict
+	    if { $fmt == "GPX" } {
+		set datum "WGS 84"
+	    } else { set datum $Datum }
+	    set sy scenic
+	    set prefix P
+	    foreach "lhs rhs" $COMMAND(params) {
+		switch $lhs {
+		    id {
+			if { [set ix [IndexNamed $type $rhs]] == -1 } {
+			    GMMessage "$MESS(notlisted): $rhs"
+			    return 1
+			}
+		    }
+		    date {
+			set mdate $rhs
+		    }
+		    datum {
+			if { [EllipsdOf $rhs] == "" } {
+			    GMMessage "$MESS(unkndatum): $rhs"
+			    return 1
+			}
+			if { $fmt != "GPX" } { set datum $rhs }
+		    }
+		    sy {
+			if { [lsearch -exact $SYMBOLS $rhs] == -1 } {
+			    GMMessage "$MESS(badSYMBOLcode): $rhs"
+			    return 1
+			}
+			set sy $rhs
+		    }
+		    prefix {
+			set prefix $rhs
+		    }
+		    __picts {
+			set pcs $rhs
+		    }
+		}
+	    }
+	    if { [set lts [GetFilesDates $pcs $mdate]] == "" || \
+		     [set lts [GeoAdapt $type $ix $datum $lts]] == "" } {
+		GMMessage $MESS(missingdata)
+		return 1
+	    }
+	    if { [CmdProcParamsFails $fmt $COMMAND(out,params)] || \
+		     [CmdGPictsOutputFails $fmt $COMMAND(path) \
+			  $lts $datum $sy $prefix] } {
+		return 1
+	    }
+	}
 	source {
 	    if { [catch {source $COMMAND(path)}] } { return 1 }
 	    return wait
 	}
+	read {
+    	    if { ! $COMMAND(script) } {
+		# enter graphical mode
+		set CMDLINE 0
+		GMInit
+		if { [CmdReadFails $COMMAND(format) $COMMAND(path) \
+			  $COMMAND(in,params) $COMMAND(filetypes)] } {
+		    GMMessage "$MESS(badfile): $COMMAND(path)"
+		}
+		return wait
+	    }
+	    return [CmdReadFails $COMMAND(format) $COMMAND(path) \
+			$COMMAND(in,params) $COMMAND(filetypes)]
+	}
 	exec {
 	    if { [catch {set f [open $COMMAND(path) r]}] } { return 1 }
 	    set COMMAND(script) 1
 	    return [CmdInterpret $f]
 	}
-	read {
-	    parray COMMAND
-	    return [CmdReadFails $COMMAND(format) $COMMAND(path) \
-		                 $COMMAND(in,params) $COMMAND(filetypes)]
-	}
 	write {
 	    return [CmdWriteFails $COMMAND(format) $COMMAND(path) \
 		                  $COMMAND(out,params) $COMMAND(filetypes)]
@@ -382,17 +450,8 @@ proc CmdWriteFails {fmt path pars types} {
 		if { [catch {set r [ExportFileTo $f all $fmt $types]}] || \
 			$r == 1 } { return 1 }
 	    } else {
-		# more than 1 type per file
-		# assume that file can be written in append mode a type at
-		#  a time
-		foreach t $types {
-		    if { [catch {set r [ExportFileTo $f all $fmt $t]}] || \
-			    $r == 1  || \
-			    [set f [CmdOpenFile $path app]] == -1 } {
-			return 1
-		    }
-		}
-		if { $path != "stdout" } { close $f }
+		set lp [AllIndicesForType Data $types]
+		if { [catch {Export_$fmt $f $lp}] } { return 1 }
 	    }
 	}
     }
@@ -465,6 +524,9 @@ proc CmdRTimeGotFix {args} {
 	puts $PVTDFile $epe
 	puts $PVTDFile $eph
 	puts $PVTDFile $epv
+	if { $fix == $TXT(posfix3D) || $fix == $TXT(posfix3D-diff) } {
+	    puts $PVTDFile $alt
+	} else { puts $PVTDFile "" }
 	close $PVTDFile
 	exit 0
     }
@@ -475,7 +537,7 @@ proc CmdRTimeGotFix {args} {
 proc CmdShow {what} {
     # print information to stdout
     global VERSION FILEFORMAT RECPROTS TXT MESS MAPKNOWNPROJS MAPPROJDATA \
-	    MAPPROJDTYPE MAPPARTPDATA MAPKNOWNTRANSFS GDATUM COMMAND
+	MAPPROJDTYPE MAPPARTPDATA MAPKNOWNTRANSFS GDATUM COMMAND SYMBOLS
 
     switch $what {
 	help { puts $COMMAND(USAGE) }
@@ -559,6 +621,14 @@ proc CmdShow {what} {
 		puts "$prot ($rec)"
 	    }
 	}
+	symbols {
+	    foreach s [lsort -dictionary $SYMBOLS] {
+		if { [string length $s] > 7 } {
+		    set sep "\t"
+		} else { set sep "\t\t" }
+		puts "$s$sep$TXT(SY$s)"
+	    }
+	}
     }
     return
 }
@@ -696,6 +766,74 @@ proc CmdGenerateGeoRef {} {
     return
 }
 
+proc CmdGPictsOutputFails {fmt path lts datum sy prefix} {
+    # write a waypoint for each tuple in $lts to file in format $fmt
+    #  $fmt must be GPX (for the time being)
+    #  $lts is list of tuples with the latitude and longitude (decimal
+    #     degrees, for $datum) and altitude (metre), date in seconds from
+    #     $YEAR0, file path, and date in "%Y:%m:%d %H:%M:%S" format
+    #  $sy the symbol code to use for WPs
+    #  $prefix is the name prefix to be used when generating WP names
+    # if  $path==stdout  use standard output
+    # return 0 on success
+
+    if { $fmt != "GPX" } { BUG CmdGPictsOutputFails: format must be GPX }
+    if { $path != "stdout" } {
+	if { [catch {set file [open $path w]}] } { return 1 }
+    } else { set file stdout }
+
+    # output GPX
+    set xml_hdr {<?xml version="1.0" encoding="ISO-8859-1" standalone="yes"?>
+  <gpx
+    version="1.0"
+    creator="GPSMan" 
+    xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
+    xmlns="http://www.topografix.com/GPX/1/0"
+    xmlns:topografix="http://www.topografix.com/GPX/Private/TopoGrafix/0/2"
+    xsi:schemaLocation="http://www.topografix.com/GPX/1/0 http://www.topografix.com/GPX/1/0/gpx.xsd http://www.topografix.com/GPX/Private/TopoGrafix/0/2 http://www.topografix.com/GPX/Private/TopoGrafix/0/2/topografix.xsd">
+    <author>anonymous GPSMan user</author>
+    <email>undefined</email>
+    <url>undefined</url>
+    <urlname>undefined</urlname>
+}
+
+    set xml_footer {</gpx>}
+
+    set xml_wptalt {<wpt lat="%s" lon="%s">
+    <name>%s</name>    <ele>%s</ele>
+    <symbol>%s</symbol>
+    <desc>%s</desc>
+    <extensions><picture>%s</picture></extensions>
+</wpt>
+}
+
+    set xml_wpt {<wpt lat="%s" lon="%s">
+    <name>%s</name>
+    <symbol>%s</symbol>
+    <desc>%s</desc>
+    <extensions><picture>%s</picture></extensions>
+</wpt>
+}
+    set n 0
+    set dpre "Photo"
+    puts $file $xml_hdr
+    foreach tuple $lts {
+	foreach "la lo alt secs fpath date" $tuple { break }
+	set name "$prefix[format %05d $n]"
+	incr n
+	set fname [file tail $fpath]
+	set dscr "$dpre $fname"
+	if { $alt != "" } {
+	    puts $file [format $xml_wptalt $la $lo $name $alt $sy $dscr $fname]
+	} else {
+	    puts $file [format $xml_wpt $la $lo $name $sy $dscr $fname]
+	}
+    }
+    puts $file $xml_footer
+    if { $path != "stdout" } { close $file }
+    return 0
+}    
+
 ##### logo window
 
 proc ShowGPSMan {} {
