@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: command_parse.tcl
-#  Last change:  5 April 2006
+#  Last change:  26 May 2007
 #
 
 # this file is only source-ed in command-line mode
@@ -37,7 +37,8 @@ set COMMAND(USAGE) {Usage: gpsman [PREFIXES] COMMAND [ARGS]
     -prot Garmin | Garmin_USB | NMEA | SText | simul   receiver protocol
   COMMANDS [ARGS]
     is available | connected
-    show help | version | formats | protocols | projections | datums | transfs
+    show help | version | formats | protocols | projections | datums |
+                transfs | symbols
     haslib gpsmanshp | Img
     getwrite IN-TYPES FORMAT [OUT-PARAMS] [OUT-TYPES] PATH
     readput FORMAT [IN-PARAMS] [IN-TYPES] PATH [OUT-TYPES]  (-rec not allowed!)
@@ -51,10 +52,20 @@ set COMMAND(USAGE) {Usage: gpsman [PREFIXES] COMMAND [ARGS]
     georef IMGPATH TRANSF LATD LONGD LATD LONGD [LATD LONGD] DATUM \
            X Y X Y [X Y] PROJECTION [PARAMS]
             (no PREFIXES; DATUM used for projection if PARAMS do not give one)
+    geopicts IN-FORMAT [IN-PARAMS] IN-TYPE IN-PATH [PROC-PARAMS] \
+	 [OUT-FORMAT] [OUT-PARAMS] OUT-PATH PICT-PATHS
+	   where
+	    IN-TYPE: TR | GR (for WPs in GR)
+            PROC-PARAMS: [id=ID] [date=DMODE]
+             ID: identifier of item in data file, default is last read
+             DMODE: pict|fmod   date from EXIF (default), or file modified time
+	    OUT-FORMAT: GPX
+	    OUT-PARAMS may include: datum=DATUM sy=SYMBOL prefix=STRING
+               STRING: is prefix for WP names, default is P
     source PATH
+    read FORMAT [PARAMS] [TYPES] PATH    if not in script launches interface
     exec PATH
        for use in scripts only:
-           read FORMAT [PARAMS] [TYPES] PATH
            write FORMAT [PARAMS] [TYPES] PATH
            get TYPES
            put TYPES
@@ -77,11 +88,13 @@ proc BadCommandLine {} {
 	    rec 0
 	    log 0
 	    prot ""
+	    command 1
 	}
     }
     set pargs 0
     while 1 {
-	switch -- [lindex $argv 0] {
+	set opt [lindex $argv 0]
+	switch -- $opt {
 	    -dev {
 		set pos 1
 		set SERIALPORT [lindex $argv 1]
@@ -96,18 +109,25 @@ proc BadCommandLine {} {
 		# not to be used with either  readput  or  -prefs
 		#  or after  -prot
 		if { $COMMAND(prefsfile) != "" || \
-			$COMMAND(prot) != "" } { return 1 }
+			$COMMAND(prot) != "" } {
+		    puts stderr "Cannot use -rec with readput, -prefs or -prot"
+		    return 1
+		}
 		set pos 4
 		set GPSREC [lindex $argv 1]
 		set argv [lreplace $argv 0 1]
 		if { [lsearch {Garmin Lowrance Magellan} $GPSREC] == -1 } {
+		    puts stderr "Bad receiver brand $GPSREC"
 		    return 1
 		}
 		set COMMAND(rec) 1
 	    }
 	    -prefs {
 		# not to be used with  -rec
-		if { $COMMAND(rec) } { return 1 }
+		if { $COMMAND(rec) } {
+		    puts stderr "Cannot use -prefs with -rec"
+		    return 1
+		}
 		set pos 8
 		set COMMAND(prefsfile) [lindex $argv 1]
 		set argv [lreplace $argv 0 1]
@@ -123,23 +143,30 @@ proc BadCommandLine {} {
 	    }
 	}
 	# no repeated options
-	if { $pargs & $pos } { return 1 }
+	if { $pargs & $pos } {
+	    puts stderr "Repeated option $opt"
+	    return 1
+	}
 	incr pargs $pos
     }
 
     set cmd [lindex $argv 0]
-    if { ( $pargs != 0 && \
+    if { $pargs != 0 && \
 	    ( $COMMAND(script) || \
-	      [lsearch -exact "project georef" $cmd] != -1 )) || \
-	   ( $cmd == "readput" && $COMMAND(rec) ) } {
+		  [lsearch -exact "project georef" $cmd] != -1 ) } {
+	puts stderr "No prefix options allowed in this command"
+	return 1
+    }
+    if { $cmd == "readput" && $COMMAND(rec) } {
+	puts stderr "Option -rec cannot be used with readput"
 	return 1
     }
-
     set argv [lreplace $argv 0 0]
     set nargs [llength $argv]
     switch -- $cmd {
 	is {
-	    switch -- [lindex $argv 0] {
+	    set what [lindex $argv 0]
+	    switch -- $what {
 		available {
 		    set COMMAND(name) available
 		    return "-0"
@@ -147,17 +174,24 @@ proc BadCommandLine {} {
 		connected {
 		    # for the time being this is only supported for
 		    #  Garmin receivers
-		    if { $GPSREC != "Garmin" } { return 1 }
+		    if { $GPSREC != "Garmin" } {
+			puts stderr "Not a Garmin receiver"
+			return 1
+		    }
 		    set cmd checkconn
 		}
-		default { return 1 }
+		default {
+		    puts stderr "Unknown argument $what; use \"show help\""
+		    return 1
+		}
 	    }
 	}
 	show {
 	    set showwhat \
-		    {help version formats projections protocols datums transfs}
+	    {help version formats projections protocols datums transfs symbols}
 	    set what [lindex $argv 0]
 	    if { [lsearch $showwhat $what] == -1 } {
+		puts stderr "Unknown argument $what; use \"show help\""
 		return 1
 	    }
 	    set COMMAND(what) $what
@@ -165,6 +199,7 @@ proc BadCommandLine {} {
 	haslib {
 	    set what [lindex $argv 0]
 	    if { [lsearch "gpsmanshp Img" $what] == -1 } {
+		puts stderr "Unknown argument $what; use \"show help\""
 		return 1
 	    }
 	    set COMMAND(what) $what
@@ -172,20 +207,26 @@ proc BadCommandLine {} {
 	getwrite {
 	    # IN-TYPES FORMAT [PARAMS] [OUT-TYPES] PATH
 	    # for the time being this is only supported for Garmin receivers
-	    if { $GPSREC != "Garmin" } { return 1 }
+	    if { $GPSREC != "Garmin" } {
+		puts stderr "Not a Garmin receiver"
+		return 1
+	    }
 	    foreach "in_ts tail" [CmdParseTypes $argv] {}
 	    if { [llength $tail] < 2 || \
 		    [set in_ts [CmdRecTypes get $in_ts]] == "" } {
+		puts stderr "Missing arguments"
 		return 1
 	    }
-	    set fmt [lindex $tail 0]
+	    if { [set fmt [CmdFFormat [lindex $tail 0]]] == "" } { return 1 }
 	    foreach "out_ps out_ts tail" \
 		[CmdParseParamsTypes [lreplace $tail 0 0]] {}
 	    if { $out_ts == "" } { set out_ts $in_ts }
-	    if { [llength $tail] != 1 || \
-		    [set COMMAND(path) $tail] == "" || \
-		    [CmdFileFmtParamsTypes out $fmt $out_ps $out_ts] == 0 || \
-		    [CmdIncompatibleTypes $in_ts $COMMAND(filetypes)] } {
+	    if { [llength $tail] != 1 || [set COMMAND(path) $tail] == "" } {
+		puts stderr "Bad output path: $tail"
+		return 1
+	    }
+	    if { [CmdFileFmtParamsTypes out $fmt $out_ps $out_ts] == 0 || \
+		     [CmdIncompatibleTypes $in_ts $COMMAND(filetypes)] } {
 		return 1
 	    }
 	    set COMMAND(rectypes) $in_ts
@@ -193,20 +234,31 @@ proc BadCommandLine {} {
 	readput {
 	    # FORMAT [PARAMS] [IN-TYPES] PATH [OUT-TYPES]
 	    # for the time being this is only supported for Garmin receivers
-	    if { $GPSREC != "Garmin" } { return 1 }
+	    if { $GPSREC != "Garmin" } {
+		puts stderr "Not a Garmin receiver"
+		return 1
+	    }
 	    # no  -rec  option
-	    if { $COMMAND(rec) } { return 1 }
-	    if { [set fmt [lindex $argv 0]] == "" } { return 1 }
+	    if { $COMMAND(rec) } {
+		puts stderr "Option -rec not allowed with readput"				return 1
+	    }
+	    if { [set fmt [CmdFFormat [lindex $argv 0]]] == "" } { return 1 }
 	    foreach "in_ps in_ts tail" \
 		[CmdParseParamsTypes [lreplace $argv 0 0]] {}
-	    if { [set COMMAND(path) [lindex $tail 0]] == "" || \
-		    [CmdFileFmtParamsTypes in $fmt $in_ps $in_ts] == 0 } {
+	    if { [set COMMAND(path) [lindex $tail 0]] == "" } {
+		puts stderr "Missing output path"
+		return 1
+	    }
+	    if { [CmdFileFmtParamsTypes in $fmt $in_ps $in_ts] == 0 } {
 		return 1
 	    }
 	    foreach "out_ts tail" [CmdParseTypes [lreplace $tail 0 0]] {}
 	    if { $out_ts == "" } { set out_ts $COMMAND(filetypes) }
-	    if { $tail != "" || \
-		    [set out_ts [CmdRecTypes put $out_ts]] == "" || \
+	    if { $tail != "" } {
+		puts stderr "Extra arguments after types"
+		return 1
+	    }
+	    if { [set out_ts [CmdRecTypes put $out_ts]] == "" || \
 		    [CmdIncompatibleTypes $COMMAND(filetypes) $out_ts] } {
 		return 1
 	    }
@@ -215,13 +267,19 @@ proc BadCommandLine {} {
 	translate {
 	    # IN-FORMAT [IN-PARAMS] [IN-TYPES] IN-PATH OUT-FORMAT \
             #    [OUT-PARAMS] [OUT-TYPES] OUT-PATH
-	    if { $nargs < 4 } { return 1 }
-	    if { [set infmt [lindex $argv 0]] == "" } { return 1 }
+	    if { $nargs < 4 } {
+		puts stderr "Missing arguments"
+		return 1
+	    }
+	    if { [set infmt [CmdFFormat [lindex $argv 0]]] == "" } { return 1 }
 	    foreach "in_ps in_ts tail" \
 		[CmdParseParamsTypes [lreplace $argv 0 0]] {}
-	    if { [set COMMAND(inpath) [lindex $tail 0]] == "" || \
-		    [CmdFileFmtParamsTypes in $infmt $in_ps $in_ts] == 0 || \
-		    [set outfmt [lindex $tail 1]] == "" } {
+	    if { [set COMMAND(inpath) [lindex $tail 0]] == "" } {
+		puts stderr "Missing input path"
+		return 1
+	    }
+	    if { [CmdFileFmtParamsTypes in $infmt $in_ps $in_ts] == 0 || \
+		     [set outfmt [CmdFFormat [lindex $tail 1]]] == "" } {
 		return 1
 	    }
 	    set COMMAND(infmt) $COMMAND(format)
@@ -229,74 +287,116 @@ proc BadCommandLine {} {
 		[CmdParseParamsTypes [lreplace $tail 0 1]] {}
 	    if { $out_ts == "" } { set out_ts $COMMAND(filetypes) }
 	    set COMMAND(intypes) $COMMAND(filetypes)
-	    if { [llength $tail] != 1 || \
-		  [set COMMAND(path) $tail] == "" || \
-		  [CmdFileFmtParamsTypes out $outfmt $out_ps $out_ts] == 0 || \
+	    if { [llength $tail] != 1 } {
+		puts stderr "Extra/missing arguments after output types"
+		return 1
+	    }
+	    if { [set COMMAND(path) $tail] == "" } {
+		puts stderr "Missing output path"
+		return 1
+	    }
+	    if { [CmdFileFmtParamsTypes out $outfmt $out_ps $out_ts] == 0 || \
 		  [CmdIncompatibleTypes $COMMAND(intypes) \
-					  $COMMAND(filetypes)] } {
+		       $COMMAND(filetypes)] } {
 	        return 1
 	    }
 	}
 	getrtimelog {
 	    # PATH [INTERVAL]
 	    # for the time being this is only supported for Garmin receivers
-	    if { $GPSREC != "Garmin" } { return 1 }
-	    if { [set COMMAND(path) [lindex $argv 0]] == "" || \
-		     $nargs > 2 || \
-		     [set COMMAND(interv) \
-			  [CmdParseInteger [lindex $argv 1] 2]] < 0 } {
+	    if { $GPSREC != "Garmin" } {
+		puts stderr "Not a Garmin receiver"
+		return 1
+	    }
+	    if { [set COMMAND(path) [lindex $argv 0]] == "" } {
+		puts stderr "Missing output path"
+		return 1
+	    }
+	    if { $nargs > 2 || [set COMMAND(interv) \
+		     [CmdParseInteger [lindex $argv 1] 2]] < 0 } {
+		puts stderr "Expecting a number in seconds"
 		return 1
 	    }
 	}
 	getfix {
 	    # PATH [TIMEOUT]
 	    # for the time being this is only supported for Garmin receivers
-	    if { $GPSREC != "Garmin" } { return 1 }
-	    if { [set COMMAND(path) [lindex $argv 0]] == "" || \
-		     $nargs > 2 || \
-		     [set COMMAND(timeout) \
-			  [CmdParseInteger [lindex $argv 1] 0]] < 0 } {
+	    if { $GPSREC != "Garmin" } {
+		puts stderr "Not a Garmin receiver"
+		return 1
+	    }
+	    if { [set COMMAND(path) [lindex $argv 0]] == "" } {
+		puts stderr "Missing output path"
+		return 1
+	    }
+	    if { $nargs > 2 || [set COMMAND(timeout) \
+				 [CmdParseInteger [lindex $argv 1] 0]] < 0 } {
+		puts stderr "Expecting a number in seconds"
 		return 1
 	    }
 	}
 	getalmanac {
 	    # PATH
 	    # for the time being this is only supported for Garmin receivers
-	    if { $GPSREC != "Garmin" } { return 1 }
-	    if { $nargs != 1 } { return 1 }
+	    if { $GPSREC != "Garmin" } {
+		puts stderr "Not a Garmin receiver"
+		return 1
+	    }
+	    if { $nargs != 1 } {
+		puts stderr "Extra/missing arguments"
+		return 1
+	    }
 	    set COMMAND(path) [lindex $argv 0]
 	}
 	start {
 	    # travel [INTERVAL]
-	    if { [lindex $argv 0] != "travel" || $GPSREC != "Garmin" || \
-		     $nargs > 2 || \
-		     [set COMMAND(interv) \
+	    if { [lindex $argv 0] != "travel" } {
+		puts stderr "Unknown argument [lindex $argv 0]"
+		return 1
+	    }
+	    if { $GPSREC != "Garmin" } {
+		puts stderr "Not a Garmin receiver"
+		return 1
+	    }
+	    if { $nargs > 2 || [set COMMAND(interv) \
 			  [CmdParseInteger [lindex $argv 1] 2]] < 0 } {
+		puts stderr "Expecting a number in seconds"
 		return 1
 	    }
 	    set cmd starttravel
+	    set COMMAND(command) 0
 	}
 	project {
 	    # LATD LONGD DATUM PROJECTION [PARAMS]
 	    # making only a preliminary check on the correction of the
 	    #  arguments as projections.tcl was not yet loaded
-	    if { $nargs < 4 } { return 1 }
+	    if { $nargs < 4 } {
+		puts stderr "Missing arguments"
+		return 1
+	    }
 	    foreach "latd longd" $argv { break }
 	    foreach p "latd longd" {
 		set x [set $p]
-		if { ! [regexp {^-?[0-9]+(\.[0-9]+)?$} $x] } { return 1 }
+		if { ! [regexp {^-?[0-9]+(\.[0-9]+)?$} $x] } {
+		    puts stderr "Bad coordinate $x"
+		    return 1
+		}
 		set COMMAND($p) $x
 	    }
 	    set i 1
 	    foreach p "ptsdatum proj" {
 		if { [set COMMAND($p) [lindex $argv [incr i]]] == "" } {
+		    puts stderr "Missing $p argument"
 		    return 1
 		}
 	    }
-	    foreach "params xx xxx" \
-		    [CmdParseParamsTypes [lrange $argv [incr i] end]] { break }
-	    if { $xx != "" || $xxx != "" } { return 1 }
-	    set COMMAND(params) $params	    
+	    foreach "params tail" \
+		    [CmdParseParams [lrange $argv [incr i] end]] { break }
+	    if { $tail != "" } {
+		puts stderr "Extra arguments after attibute=value pairs"
+		return 1
+	    }
+	    set COMMAND(params) $params
 	}
 	georef {
 	    # IMGPATH TRANSF LATD LONGD LATD LONGD [LATD LONGD] DATUM
@@ -305,8 +405,14 @@ proc BadCommandLine {} {
 	    #  arguments as projections.tcl was not yet loaded
 	    set COMMAND(image) [lindex $argv 0]
 	    set COMMAND(transf) [set tr [lindex $argv 1]]
-	    if { $nargs < 12 || \
-		    [lsearch -exact $MAPKNOWNTRANSFS $tr] == -1 } { return 1 }
+	    if { $nargs < 12 } {
+		puts stderr "Missing arguments"
+		return 1
+	    }
+	    if { [lsearch -exact $MAPKNOWNTRANSFS $tr] == -1 } {
+		puts stderr "Unknown transformation $tr"
+		return 1
+	    }
 	    if { $tr == "NoRot" } {
 		# not using 3 points as when doing this by hand
 		set npts 2
@@ -316,60 +422,142 @@ proc BadCommandLine {} {
 	    for { set p 0 } { $p < $npts } { incr p } {
 		foreach pp "lat long" {
 		    set x [lindex $argv [incr i]]
-		    if { ! [regexp {^-?[0-9]+(\.[0-9]+)?$} $x] } { return 1 }
+		    if { ! [regexp {^-?[0-9]+(\.[0-9]+)?$} $x] } {
+			puts stderr "Bad coordinate $x"
+			return 1
+		    }
 		    set COMMAND($p,$pp) $x
 		}
 	    }
 	    if { [set COMMAND(ptsdatum) [lindex $argv [incr i]]] == "" } {
+		puts stderr "Missing ptsdatum argument"
 		return 1
 	    }
 	    for { set p 0 } { $p < $npts } { incr p } {
 		foreach pp "x y" {
 		    set x [lindex $argv [incr i]]
-		    if { ! [regexp {^-?[0-9]+(\.[0-9]+)?$} $x] } { return 1 }
+		    if { ! [regexp {^-?[0-9]+(\.[0-9]+)?$} $x] } {
+			puts stderr "Bad coordinate $x"
+			return 1
+		    }
 		    set COMMAND($p,$pp) $x
 		}
 	    }
 	    if { [set COMMAND(proj) [lindex $argv [incr i]]] == "" } {
+		puts stderr "Missing projection"
 		return 1
 	    }
-	    foreach "params xx xxx" \
-		    [CmdParseParamsTypes [lrange $argv [incr i] end]] { break }
-	    if { $xx != "" || $xxx != "" } { return 1 }
+	    foreach "params tail" \
+		    [CmdParseParams [lrange $argv [incr i] end]] { break }
+	    if { $tail != "" } {
+		puts stderr "Extra arguments after attibute=value pairs"
+		return 1
+	    }
+	    set COMMAND(params) $params
+	}
+	geopicts {
+	    # IN-FORMAT [IN-PARAMS] [IN-TYPE] IN-PATH [PROC-PARAMS] \
+	    #   [OUT-FORMAT] [OUT-PARAMS] OUT-PATH PICT-PATHS
+	    # see proc CmdBadGeoPictsParams for PROC-PARAMS and OUT-PARAMS
+	    #     IN-TYPE: TR | GR (for WPs in GR)
+	    #     OUT-FORMAT: GPX
+	    if { $nargs < 4 } {
+		puts stderr "Missing arguments"
+		return 1
+	    }
+	    if { [set infmt [CmdFFormat [lindex $argv 0]]] == "" } { return 1 }
+	    foreach "in_ps in_ts tail" \
+		[CmdParseParamsTypes [lreplace $argv 0 0]] {}
+	    if { [CmdFileFmtParamsTypes in $infmt $in_ps $in_ts TR GR] == 0 } {
+		return 1
+	    }
+	    if { [set COMMAND(inpath) [lindex $tail 0]] == "" } {
+		puts stderr "No input path"
+		return 1
+	    }
+	    set COMMAND(infmt) $COMMAND(format)
+	    set COMMAND(intypes) $COMMAND(filetypes)
+	    set tail [lreplace $tail 0 0]
+	    foreach "params tail" [CmdParseParams $tail] { break }
+	    if { [set outfmt [CmdFFormat [lindex $tail 0]]] != "GPX" } {
+		puts stderr "Output format for geopicts must be GPX"
+		return 1
+	    }
+	    foreach "out_ps tail" [CmdParseParams [lreplace $tail 0 0]] {}
+	    foreach "out_ps params" \
+		     [CmdBadGeoPictsParams $params $out_ps] { break }
+	    if { $out_ps == "error" || \
+		     [CmdFileFmtParamsTypes out $outfmt $out_ps ""] == 0 } {
+		return 1
+	    }
+	    if { [set COMMAND(path) [lindex $tail 0]] == "" } {
+		puts stderr "No output path"
+		return 1
+	    }
+	    if { [set picts [lreplace $tail 0 0]] == "" } {
+		puts stderr "No picture files given"
+		return 1
+	    }
+	    lappend params __picts $picts
 	    set COMMAND(params) $params
 	}
 	source {
 	    # PATH
-	    if { $nargs != 1 } { return 1 }
-	    set COMMAND(path) $argv
-	}
-	exec {
-	    # PATH
-	    if { $nargs != 1 } { return 1 }
+	    if { $nargs != 1 } {
+		puts stderr "Missing/extra arguments"
+		return 1
+	    }
 	    set COMMAND(path) $argv
 	}
 	read {
 	    # FORMAT [PARAMS] [TYPES] PATH
-	    # for use in scripts only
-	    if { ! $COMMAND(script) || \
-		    [set fmt [lindex $argv 0]] == "" } { return 1 }
-	    foreach "in_ps in_ts tail" \
-		[CmdParseParamsTypes [lreplace $argv 0 0]] {}
-	    if { [set COMMAND(path) [lindex $tail 0]] == "" || \
-		    [CmdFileFmtParamsTypes in $fmt $in_ps $in_ts] == 0 } {
+	    # if not in a script, lauches graphical interface after reading
+	    if { [set fmt [CmdFFormat [lindex $argv 0]]] == "" } {
+		puts stderr "Missing file format"
 		return 1
 	    }
+	    foreach "in_ps in_ts tail" \
+		[CmdParseParamsTypes [lreplace $argv 0 0]] {}
+	    if { [set COMMAND(path) [lindex $tail 0]] == "" } {
+		puts stderr "Missing input path"
+		return 1
+	    }
+	    if { [CmdFileFmtParamsTypes in $fmt $in_ps $in_ts] == 0 } {
+		return 1
+	    }
+	    if { ! $COMMAND(script) } {
+		set COMMAND(command) 0
+		# make path absolute to avoid problems with cd's
+		set COMMAND(path) [file normalize $COMMAND(path)]
+	    }
+	}
+	exec {
+	    # PATH
+	    if { $nargs != 1 } {
+		puts stderr "Missing/extra arguments"
+		return 1
+	    }
+	    set COMMAND(path) $argv
 	}
 	write {
 	    # FORMAT [PARAMS] [TYPES] PATH
 	    # for use in scripts only
-	    if { ! $COMMAND(script) } { return 1 }
-	    set fmt [lindex $argv 0]
+	    if { ! $COMMAND(script) } {
+		puts stderr "Command to be used only in a script"
+		return 1
+	    }
+	    if { [set fmt [CmdFFormat [lindex $argv 0]]] == "" } { return 1 }
 	    foreach "out_ps out_ts tail" \
 		    [CmdParseParamsTypes [lreplace $argv 0 0]] {}
-	    if { [llength $tail] != 1 || \
-		    [set COMMAND(path) $tail] == "" || \
-		    [CmdFileFmtParamsTypes out $fmt $out_ps $out_ts] == 0 } {
+	    if { [llength $tail] != 1 } {
+		puts stderr "Missing/extra arguments before path"
+		return 1
+	    }
+	    if { [set COMMAND(path) $tail] == "" } {
+		puts stderr "Missing output path"
+		return 1
+	    }
+	    if { [CmdFileFmtParamsTypes out $fmt $out_ps $out_ts] == 0 } {
 		return 1
 	    }
 	}
@@ -377,23 +565,49 @@ proc BadCommandLine {} {
 	    # TYPES
 	    # for use in scripts only
 	    # for the time being this is only supported for Garmin receivers
-	    if { ! $COMMAND(script) || $GPSREC != "Garmin" } { return 1 }
+	    if { ! $COMMAND(script) } {
+		puts stderr "Command to be used only in a script"
+		return 1
+	    }
+	    if { $GPSREC != "Garmin" } {
+		puts stderr "Not a Garmin receiver"
+		return 1
+	    }
 	    foreach "in_ts tail" [CmdParseTypes $argv] {}
-	    if { $tail != "" } { return 1 }
+	    if { $tail != "" } {
+		puts stderr "Extra arguments after types"
+		return 1
+	    }
 	    set COMMAND(rectypes) $in_ts
 	}
 	put {
 	    # TYPES
 	    # for use in scripts only
 	    # for the time being this is only supported for Garmin receivers
-	    if { ! $COMMAND(script) || $GPSREC != "Garmin" } { return 1 }
+	    if { ! $COMMAND(script) } {
+		puts stderr "Command to be used only in a script"
+		return 1
+	    }
+	    if { $GPSREC != "Garmin" } {
+		puts stderr "Not a Garmin receiver"
+		return 1
+	    }
 	    # no  -rec  option
-	    if { $COMMAND(rec) } { return 1 }
+	    if { $COMMAND(rec) } {
+		puts stderr "Option -rec not allowed with put"
+		return 1
+	    }
 	    foreach "out_ts tail" [CmdParseTypes $argv] {}
-	    if { $tail != "" } { return 1 }
+	    if { $tail != "" } {
+		puts stderr "Extra arguments after types"
+		return 1
+	    }
 	    set COMMAND(rectypes) $out_ts
 	}
-	default { return 1 }
+	default {
+	    puts stderr "Bad command name $cmd; use \"show help\""
+	    return 1
+	}
     }
     set COMMAND(name) $cmd
     return 0
@@ -404,27 +618,46 @@ proc CmdParseProt {prot} {
     # return "" on error
     global RECPROTS GPSREC
 
-    if { [catch {set p $RECPROTS($GPSREC,$prot)}] } { return "" }
+    if { [catch {set p $RECPROTS($GPSREC,$prot)}] } {
+	puts stderr "Bad receiver protocol $prot"
+	return ""
+    }
     return $p
 }
 
 proc CmdParseTypes {lst} {
     # find element(s) at the beginning of $lst corresponding to item types:
-    #   either a single "all", or a subset of {WP, RT, TR, LN, LAP}
+    #   either a single "all", or a subset of {WP, RT, TR, LN, LAP, GR}
     #   LAP is accepted only if $SUPPORTLAPS is set
+    #   GR will be accepted later only if the format is "GPSMan"
     #   (note that $TYPES is not defined yet!)
-    # return pair with list of those elements and rest of list
+    # return pair with list of types and rest of list
 
     set pstslst [CmdParseParamsTypes $lst]
     if { [lindex $pstslst 0] != "" } { return [list "" $lst] }
     return [lreplace $pstslst 0 0]
 }
 
+proc CmdParseParams {lst} {
+    # find element(s) at the beginning of $lst corresponding to
+    #  parameters in the form NAME=VALUE
+    # return pair with list of parameters (with alternating lhs and rhs),
+    #  and rest of list
+
+    set ps ""
+    while { [regexp {^(.+)=(.+)$} [lindex $lst 0] x lhs rhs] } {
+	lappend ps $lhs $rhs
+	set lst [lreplace $lst 0 0]
+    }
+    return [list $ps $lst]
+}
+
 proc CmdParseParamsTypes {lst} {
     # find element(s) at the beginning of $lst corresponding to
     #  parameters in the form NAME=VALUE, followed by item types:
-    #  either a single "all", or a subset of {WP, RT, TR, LAP, LN}
-    #   LAP is accepted only if $SUPPORTLAPS is set
+    #  either a single "all", or a subset of {WP, RT, TR, LAP, LN, GR}
+    #   LAP is accepted here only if $SUPPORTLAPS is set
+    #   GR will be accepted later only if the format is "GPSMan"
     #  (note that $TYPES is not defined yet!)
     # return triple with list of parameters (with alternating lhs and rhs),
     #  list of types and rest of list
@@ -440,8 +673,8 @@ proc CmdParseParamsTypes {lst} {
     } else {
 	set ts ""
 	if { $SUPPORTLAPS } {
-	    set ats "WP RT TR LN LAP"
-	} else { set ats "WP RT TR LN" }
+	    set ats "WP RT TR LN LAP GR"
+	} else { set ats "WP RT TR LN GR" }
 	while { [lsearch -exact $ats [lindex $lst 0]] != -1 } {
 	    lappend ts [lindex $lst 0] ; set lst [lreplace $lst 0 0]
 	}
@@ -463,11 +696,11 @@ proc CmdParseInteger {val default} {
 proc CmdRecTypes {how lst} {
     # interpret list of item types to be get/put from/into receiver
     #  $how in {get, put}
-    #  $lst may be either "all", or non-empty subset of $TYPES-{LN,GR}
+    #  $lst may be either "all", or non-empty subset of $TYPES-{LN, GR}
     #  LAP is accepted only if $SUPPORTLAPS is set and $how=="get"
     #  (note that $TYPES is not defined yet!)
     # return "" on error, otherwise list of types as non-empty subset
-    #  of $TYPES-{LN,GR}
+    #  of $TYPES-{LN, GR}
     global SUPPORTLAPS
 
     if { $lst == "all" } {
@@ -478,10 +711,14 @@ proc CmdRecTypes {how lst} {
 	foreach t $lst {
 	    switch -- $t {
 		LN -  GR {
+		    puts stderr "Cannot get/put LNs or GRs"
 		    return ""
 		}
 		LAP {
-		    if { ! $SUPPORTLAPS || $how == "put" } { return "" }
+		    if { ! $SUPPORTLAPS || $how == "put" } {
+			puts stderr "LAP not supported; check the preferences!"
+			return ""
+		    }
 		}
 	    }
 	}
@@ -489,13 +726,29 @@ proc CmdRecTypes {how lst} {
     return $lst
 }
 
-proc CmdFileFmtParamsTypes {how fmt pars ts} {
+proc CmdFFormat {fmt} {
+    # check that $fmt is a valid name for a file format, as
+    #  used in the FILEFORMAT array or all in lowercase
+    # return name as used in FILEFORMAT or "" on error
+    global FILEFORMAT
+
+    foreach n $FILEFORMAT(names) {
+	if { $fmt == $n || $fmt == [string tolower $n] } {
+	    return $n
+	}
+    }
+    puts stderr "Unknown file format name: $fmt"
+    return ""
+}
+
+proc CmdFileFmtParamsTypes {how fmt pars ts args} {
     # check that $fmt is a valid file format for I/O operation on file
     #  with given parameters (values not checked here) and item types
-    # if $fmt=="gpsman" it is taken to be "GPSMan"
+    #  and, if $args!="", that item types are in $args or default to a
+    #  subset of them
     #  $how in {in, out}
     #  $pars is list with each parameter name followed by corresponding value
-    #  $ts is either "all", or subset of $TYPES-{GR}
+    #  $ts is either "all", or subset of $TYPES
     #     LAP is only accepted if $SUPPORTLAPS is set
     # set COMMAND(filetypes), COMMAND(format), COMMAND($how,params)
     #  the latter in the same format as $pars but with all parameters in
@@ -503,10 +756,8 @@ proc CmdFileFmtParamsTypes {how fmt pars ts} {
     # return 0 on error
     global FILEFORMAT COMMAND SUPPORTLAPS
 
-    # compatibility with old versions
-    if { $fmt == "gpsman" } {
-	set fmt GPSMan
-    } elseif { [regexp {^Shapefile_([23])D$} $fmt x dim] } {
+    # compatibility with older versions
+    if { [regexp {^Shapefile_([23])D$} $fmt x dim] } {
 	set fmt Shapefile
 	lappend pars dim $dim
     }
@@ -579,6 +830,14 @@ proc CmdFileFmtParamsTypes {how fmt pars ts} {
     if { [lsearch -exact $fts LAP] != -1 && ! $SUPPORTLAPS } {
 	return 0
     }
+    if { $args != "" } {
+	foreach t $fts {
+	    if { [lsearch $args $t] == -1 } {
+		puts stderr "Bad type $t; should be one of $args"
+		return 0
+	    }
+	}
+    }
     set COMMAND(format) $fmt
     set COMMAND($how,params) $pars
     set COMMAND(filetypes) $fts
@@ -587,15 +846,72 @@ proc CmdFileFmtParamsTypes {how fmt pars ts} {
 
 proc CmdIncompatibleTypes {in_ts out_ts} {
     # check compatibility of input and output types
-    # the only possible "conversion" is from RT to WP
+    # the only possible "conversion" is from RT to WP, and GR to anything
     # return 1 on error
 
-    if { $in_ts == $out_ts } { return 0 }
+    if { $in_ts == $out_ts || [lsearch $in_ts GR] != -1 } { return 0 }
     foreach o $out_ts {
 	if { [lsearch $in_ts $o] == -1 && \
 		( $o != "WP" || [lsearch $in_ts RT] == -1 ) } {
+	    puts stderr "Output type $o incompatible with input type(s) $in_ts"
 	    return 1
 	}
     }
     return 0
-}
\ No newline at end of file
+}
+
+proc CmdBadGeoPictsParams {pps outps} {
+    # check the processing- and output-parameters of the geopict command
+    # the output-parameters may contain paramaters needed by the output format
+    #  PROC-PARAMS: [id=ID] [date=DMODE]
+    #     ID: identifier of item in data file, default is last read in
+    #     DMODE: pict | fmod   date from EXIF (default), or file modified time
+    #  OUT-PARAMS: apart from those needed by $fmt
+    #              [datum=DATUM] [sy=SYMBOL] [prefix=STRING]
+    #               STRING: defaults to "P"
+    # return "error" or pair with list of format parameters and
+    #  list with all other parameters
+
+    set ops ""
+    foreach "lhs rhs" $pps {
+	switch -- $lhs {
+	    id {
+		if { $rhs == "" } {
+		    puts stderr "Empty item identifier"
+		    return error
+		}
+	    }
+	    date {
+		if { $rhs != "fmod" && $rhs != "pict" } {
+		    puts stderr "Bad mode for date; must be fmod or pict"
+		    return error
+		}
+	    }
+	    default {
+		puts stderr "Unknown parameter $lhs"
+		return error
+	    }
+	}
+	lappend ops $lhs $rhs
+    }
+    set fps ""
+    foreach "lhs rhs" $outps {
+	# all unrecognised parameters are left as format paramters
+	switch -- $lhs {
+	    datum -  sy -  prefix {
+		# only possible check at this stage
+		if { $rhs == "" } {
+		    puts stderr "Empty argument to $lhs"
+		    return error
+		}
+	    }
+	    default {
+		lappend fps $lhs $rhs
+		continue
+	    }
+	}
+	lappend ops $lhs $rhs
+    }
+    return [list $fps $ops]
+}
+
