@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: config.tcl
-#  Last change:  4 May 2006
+#  Last change:  25 July 2006
 #
 
 ## the following values can be set from the options dialog of the interface
@@ -319,3 +319,27 @@ array set COUTFMT {
     deg    %.5f
     GRA    %.5f
 }
+
+  # options used when importing Kismet .network files
+  #   types: which Kismet network types should be converted to waypoints
+  #   esymbols: symbols (icons) to use for each encryption under each type
+  #    list of pairs with type and list of encryption followed by symbols
+  #    Tcl glob patterns can be used for the encryption
+  #     symbol names must belong to the list SYMBOLS in GPSMan file symbols.tcl
+  #   defsymbol: default symbol for encryption values not covered
+  #   prename: which prefix to use for names if the ssid is missing or name
+  #    is repeated
+  #   namenumber: initial number to add to the prefix when forming name; names
+  #    created in this way will only be used if they are acceptable, otherwise
+  #    a default name generation procedure will be used
+array set KISMETOPT {
+    types    {infrastructure ad-hoc}
+    esymbols {{infrastructure {None tall_tower * radio_beacon}}
+              {ad-hoc {* beacon}}}
+    defsymbol beacon
+    prename   KNET
+    namenumber   1
+}
+
+
+
