@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,64 +17,98 @@
 #      along with this program.
 #
 #  File: datumell.tcl
-#  Last change:  16 October 2004
+#  Last change:  20 March 2006
 #
 
-## user-defined datums and ellipsoids
+## user-definitions: datums, ellipsoids, geo-reference files
 
-proc DefineDatum {} {
-    # create dialog for defining a new datum
+# first field must be for identifier (unique for each definition type)
+#  so that  *,dfields does not include it
 
-    Definition datum \
-	    {|name |ellpsd =dx =dy =dz reserved |rmrk =ex =ey =ez "=# sat" \
-	       |latS |latN |longW |longE} \
-	    {entry >FillEllipsoidMenu float float float reserved text \
-	      poss:float>-1 poss:float>-1 poss:float>-1 poss:float>-1 \
-	      poss:lat poss:lat poss:long poss:long} \
-	    {} {create revert cancel}
-    return
+array set DefSpecs {
+    datum,dfields  {ellpsd dx dy dz reserved nb ex ey ez nsat
+	latS latN longW longE}
+    datum,fnames  {|name |ellpsd =dx =dy =dz reserved |rmrk =ex =ey =ez
+	"=# sat" |latS |latN |longW |longE}
+    datum,ftypes  {entry >FillEllipsoidMenu float float float reserved text
+	poss:float>-1 poss:float>-1 poss:float>-1 poss:float>-1
+	poss:lat poss:lat poss:long poss:long}
+    datum,datarr    GDATUM
+    datum,ulist     UDatums
+    datum,savecomm  SaveUserDatums
+    ellpsd,dfields  {a invf nb}
+    ellpsd,fnames   {|name =a =1/f |rmrk}
+    ellpsd,ftypes   {entry float>0 float>0 text}
+    ellpsd,datarr   ELLPSDDEF
+    ellpsd,ulist    UEllipsoids
+    ellpsd,savecomm SaveUserDatums
+    backgrnd,dfields {file nb}
+    backgrnd,fnames {|name |file |rmrk}
+    backgrnd,ftypes {entry file=MapBkInfo text}
+    backgrnd,datarr BACKGRNDS
+    backgrnd,ulist  UBackgrnds
+    backgrnd,savecomm SaveUserBackgrnds
 }
 
-proc DefineEllipsoid {} {
-    # create dialog for defining a new ellipsoid
+proc Define {dtype args} {
+    # create a dialog for a new definition
+    #  $dtype in {datum, ellpsd, backgrnd}
+    #  $args if defined is taken as the list of initial field values
+    # return name of new definition or "" if operation is cancelled
+    global DefSpecs
 
-    Definition ellpsd {|name =a =1/f |rmrk} {entry float>0 float>0 text} \
-	    {} {create revert cancel}
-    return
+    return [Definition $dtype $DefSpecs($dtype,fnames) \
+		$DefSpecs($dtype,ftypes) $args {create revert cancel}]
 }
 
-proc OpenDatum {datum args} {
-    # create dialog for inspecting/editing $datum
-    #  $args is the name of the menu from which this proc was called (not used)
-    global UDatums GDATUM
+proc Inspect {dtype name args} {
+    # create dialog for inspecting/editing a definition
+    #  $dtype in {datum, ellpsd, backgrnd}
+    global DefSpecs
 
-    if { [lsearch -exact $UDatums $datum] == -1 } {
-	set as {create revert cancel}
-    } else {
-	set as {change revert create forget cancel}
+    set datarr $DefSpecs($dtype,datarr) ; set ulist $DefSpecs($dtype,ulist)
+    global $datarr $ulist
+    set as {change revert create forget cancel}
+    if { $dtype != "backgrnd" } {
+	# there are GPSMan definitions apart from user ones
+	if { [lsearch -exact [set $ulist] $name] == -1 } {
+	    set as {create revert cancel}
+	}
     }
-    set d [linsert $GDATUM($datum) 0 $datum]
-    Definition datum \
-	    {|name |ellpsd =dx =dy =dz reserved |rmrk =ex =ey =ez "=# sat" \
-	       |latS |latN |longW |longE} \
-	    {entry >FillEllipsoidMenu float float float reserved text \
-	      poss:float>-1 poss:float>-1 poss:float>-1 poss:float>-1 \
-	      poss:lat poss:lat poss:long poss:long} $d $as
+    set d [linsert [set [set datarr]($name)] 0 $name]
+    Definition $dtype $DefSpecs($dtype,fnames) $DefSpecs($dtype,ftypes) $d $as
     return
 }
 
-proc OpenEllipsoid {ell args} {
-    # create dialog for inspecting/editing ellipsoid $ell
-    #  $args is the name of the menu from which this proc was called (not used)
-    global UEllipsoids ELLPSDDEF
+proc GetDefFields {dtype name fs} {
+    # return list of field values for definition of given type and name
+    #  $fs is list of field names (as in DefSpecs array)
+    # result is empty if definition does not exist; unknown fields are
+    #  ignored silently
+    global DefSpecs
 
-    if { [lsearch -exact $UEllipsoids $ell] == -1 } {
-	set as {create revert cancel}
-    } else {
-	set as {change revert create forget cancel}
+    set datarr $DefSpecs($dtype,datarr)
+    global $datarr
+    if { [catch {set data [set [set datarr]($name)]}] } { return "" }
+    set res "" ; set dflds $DefSpecs($dtype,dfields)
+    foreach f $fs {
+	if { [set ix [lsearch -exact $dflds $f]] != -1 } {
+	    lappend res [lindex $data $ix]
+	}
     }
-    set d [linsert $ELLPSDDEF($ell) 0 $ell]
-    Definition ellpsd {|name =a =1/f |rmrk} {entry float>0 float>0 text} $d $as
+    return $res
+}
+
+proc FillBackgrndMenu {menu commd args} {
+    # if $args is not empty it is a list of menu entries to be added to
+    #  the beginning of the menu
+    global UBackgrnds
+
+    if { [regexp {@|(---)} $UBackgrnds] } {
+	set l $args
+	foreach x $UBackgrnds { lappend l " $x" }
+    } else { set l [concat $args $UBackgrnds] }
+    FillMenu $menu $commd $l
     return
 }
 
@@ -102,7 +136,7 @@ proc SaveUserDatums {} {
     WriteQuoteList $f $UDatums
     puts $f "\}"
     puts $f ""
-    if { "$UDatums" != "" } {
+    if { $UDatums != "" } {
 	puts $f "array set GDATUM \{"
 	foreach d $UDatums {
 	    WriteQuote $f $d
@@ -123,7 +157,7 @@ proc SaveUserDatums {} {
     WriteQuoteList $f $UEllipsoids
     puts $f "\}"
     puts $f ""
-    if { "$UEllipsoids" != "" } {
+    if { $UEllipsoids != "" } {
 	puts $f "array set ELLPSDDEF \{"
 	foreach d $UEllipsoids {
 	    WriteQuote $f $d
@@ -140,11 +174,57 @@ proc SaveUserDatums {} {
     return
 }
 
+proc SetUpUserBackgrnds {} {
+    # process information on user-defined map backgrounds
+    global UFile BACKGRNDS UBackgrnds
+
+    source $UFile(backgrs)
+    return
+}
+
+proc SaveUserBackgrnds {} {
+    # save user-defined backgrounds
+    global UFile BACKGRNDS UBackgrnds MESS
+
+    if { [catch {set f [open $UFile(backgrs) w]}] } {
+	GMMessage $MESS(cantwrtmbaks)
+	return
+    }
+    puts $f ""
+    puts $f "# $MESS(written) GPSMan [NowTZ]"
+    puts $f "# $MESS(editrisk)"
+    puts $f ""
+    puts -nonewline $f "set UBackgrnds \{"
+    WriteQuoteList $f $UBackgrnds
+    puts $f "\}"
+    puts $f ""
+    if { $UBackgrnds != "" } {
+	puts $f "array set BACKGRNDS \{"
+	foreach d $UBackgrnds {
+	    WriteQuote $f $d
+	    set l $BACKGRNDS($d)
+	    puts -nonewline $f " \{"
+	    WriteQuote $f [lindex $l 0]
+	    puts $f ""
+	    puts -nonewline $f "    [lindex $l 1] "
+	    WriteQuote $f [lindex $l 2]
+	    puts $f "\}"
+	    puts $f ""
+	}
+	puts $f "\}"
+	puts $f ""
+    }
+    close $f
+    return
+}
+
 proc Definition {kind fields types data acts} {
     # open a dialog for creating/inspecting/editing a definition of an
     #  object
     #  $kind  must be an index of TXT, and is used as an argument to
     #    proc DefAction that must be updated for each new kind
+    # return name of new/changed definition or "" if operation is cancelled
+    #  or definition is forgotten
     # the dialog will contain entries/menus for the name of the defined
     #  object and its parameters; in the next 3 lists the name corresponds
     #  always to the first element; these 3 lists are all aligned:
@@ -157,6 +237,9 @@ proc Definition {kind fields types data acts} {
     #       poss:EL field that can be either void or the description EL
     #       "entry" for an entry with any text (but the name cannot be void)
     #       "text"  for any text (but the name cannot be void)
+    #       "file=FTYPE"  for a file path (entry and select button) of type
+    #           FTYPE (see $filetypes in proc GMStart, main.tcl); can be void
+    #           or refer to a non-existing file
     #       >PROC   for a value in a menu filled in by PROC that will be
     #           called with two arguments: the menu and the command to be
     #           invoked by a selection; this command will have two arguments:
@@ -167,14 +250,14 @@ proc Definition {kind fields types data acts} {
     #  $acts are the possible actions allowed in the set:
     #    {cancel, create, change, revert, forget}
     # the dialog grabs the focus but cannot be kept raised because of menus
-    global Datell EPOSX EPOSY COLOUR MESS TXT OBSWIDTH OBSHEIGHT
+    global Datell EPOSX EPOSY COLOUR MESS TXT OBSWIDTH OBSHEIGHT DefCommand
 
     foreach x "kind types data" {
 	set Datell($x) [set $x]
     }
     set Datell(grabs) [grab current]
     toplevel .datell
-    wm protocol .datell WM_DELETE_WINDOW { DefCommand cancel }
+    wm protocol .datell WM_DELETE_WINDOW { set DefCommand cancel }
     wm title .datell "$TXT($kind)/GPS Manager"
     wm transient .datell
     wm geometry .datell +$EPOSX+$EPOSY
@@ -212,6 +295,16 @@ proc Definition {kind fields types data acts} {
 		$vw insert 1.0 $d
 		TextBindings $vw
 	    }
+	    file=* {
+		regsub {file=} $t "" ftype
+		frame $vw
+		entry $vw.fn -width 50
+		ShowTEdit $vw.fn $d 1
+		button $vw.sel -text $TXT(select) \
+		    -command "DefSelectFile $ftype $vw"
+		grid $vw.fn -column 0 -row 0 -sticky nesw
+		grid $vw.sel -column 1 -row 0 -sticky e
+	    }
 	    >* {
 		set mn $vw.mn
 		menubutton $vw -text $d -relief raised -direction below \
@@ -233,7 +326,7 @@ proc Definition {kind fields types data acts} {
     foreach e $acts {
 	button $frb.b$e -text $TXT($e) \
 		-command "$frb.b$e configure -state normal ; \
-		          DefCommand $e"
+		          set DefCommand $e"
 	pack $frb.b$e -side left
     }
 
@@ -243,71 +336,85 @@ proc Definition {kind fields types data acts} {
     update idletasks
     # cannot use RaiseWindow because of menus
     grab .datell
-    return
-}
-
-proc DefCommand {act} {
-    # execute command called from the definitions dialog
-    global Datell
-
-    set frs .datell.fr.frsel
-    switch $act {
-	cancel {
-	    DestroyRGrabs .datell $Datell(grabs)
-	}
-	revert {
-	    set n 0
-	    foreach t $Datell(types) d $Datell(data) {
-		set vw $frs.v$n
-		regsub {poss:} $t "" t
-		switch -glob $t {
-		    text {
-			$vw delete 1.0 end ; $vw insert 1.0 $d
-		    }
-		    >* {
-			$vw configure -text $d
-		    }
-		    reserved {
-		    }
-		    entry -
-		    default {
-			$vw delete 0 end ; $vw insert 0 $d
+    while 1 {
+	tkwait variable DefCommand
+	switch $DefCommand {
+	    cancel {
+		DestroyRGrabs .datell $Datell(grabs)
+		return ""
+	    }
+	    revert {
+		set n 0
+		foreach t $Datell(types) d $Datell(data) {
+		    set vw $frs.v$n
+		    regsub {poss:} $t "" t
+		    switch -glob $t {
+			text {
+			    $vw delete 1.0 end ; $vw insert 1.0 $d
+			}
+			>* {
+			    $vw configure -text $d
+			}
+			reserved {
+			}
+			file=* {
+			    $vw.fn delete 0 end ; $vw.fn insert 0 $d
+			}
+			entry -
+			default {
+			    $vw delete 0 end ; $vw insert 0 $d
+			}
 		    }
+		    incr n
 		}
-		incr n
 	    }
-	}
-	create {
-	    set nd [DefData $Datell(kind) $frs $Datell(types)]
-	    if { [regexp {^[0-9]+$} $nd] } {
-		focus $frs.v$nd
-		return
+	    create {
+		set nd [DefData $Datell(kind) $frs $Datell(types)]
+		if { [regexp {^[0-9]+$} $nd] } {
+		    focus $frs.v$nd
+		    continue
+		}
+		set name [lindex $nd 0]
+		if { [DefAction create $Datell(kind) $name "" \
+			  [lreplace $nd 0 0]] } {
+		    DestroyRGrabs .datell $Datell(grabs)
+		    return $name
+		}
 	    }
-	    set name [lindex $nd 0]
-	    if { [DefAction $act $Datell(kind) $name "" [lreplace $nd 0 0]] } {
-		DestroyRGrabs .datell $Datell(grabs)
+	    change {
+		set nd [DefData $Datell(kind) $frs $Datell(types)]
+		if { [regexp {^[0-9]+$} $nd] } {
+		    focus $frs.v$nd
+		    continue
+		}
+		set name [lindex $nd 0] ; set oldname [lindex $Datell(data) 0]
+		if { [DefAction change $Datell(kind) $name $oldname \
+			  [lreplace $nd 0 0]] } {
+		    DestroyRGrabs .datell $Datell(grabs)
+		    return $name
+		}
 	    }
-	}
-	change {
-	    set nd [DefData $Datell(kind) $frs $Datell(types)]
-	    if { [regexp {^[0-9]+$} $nd] } {
-		focus $frs.v$nd
-		return
-	    }
-	    set name [lindex $nd 0] ; set oldname [lindex $Datell(data) 0]
-	    if { [DefAction $act $Datell(kind) $name $oldname \
-		    [lreplace $nd 0 0]] } {
-		DestroyRGrabs .datell $Datell(grabs)
-	    }
-	}
-	forget {
-	    if { [DefAction forget $Datell(kind) "" \
-		    [lindex $Datell(data) 0] ""] } {
-		DestroyRGrabs .datell $Datell(grabs)
+	    forget {
+		if { [DefAction forget $Datell(kind) "" \
+			  [lindex $Datell(data) 0] ""] } {
+		    DestroyRGrabs .datell $Datell(grabs)
+		    return ""
+		}
 	    }
 	}
     }
-    update idletasks
+    # not used
+    return
+}
+
+proc DefSelectFile {ftype fr} {
+    # get absolute file path and set appropriate entry in frame $fr
+    global TXT File
+
+    if { [set fn [GMGetFileName $TXT(select) $ftype r {} {}]] != ".." } {
+	$fr.fn delete 0 end
+	$fr.fn insert 0 $File($ftype)
+    }
     return
 }
 
@@ -322,7 +429,7 @@ proc DefData {kind frs types} {
     # collect and check data in the definition dialog
     #  $kind gives the kind of object being defined (used in error messages)
     #  $frs is the frame with the widgets containing the data
-    #  $types as in proc Define
+    #  $types as in proc Definition
     # return list with values or an integer on error giving the number of the
     #  widget (from 0) with bad value
     global TXT MESS
@@ -334,7 +441,7 @@ proc DefData {kind frs types} {
 	switch -glob $t {
 	    entry {
 		set d [string trim [$vw get]]
-		if { $n == 0 && "$d" == "" } {
+		if { $n == 0 && $d == "" } {
 		    GMMessage $MESS(namevoid)
 		    return 0
 		}
@@ -342,6 +449,9 @@ proc DefData {kind frs types} {
 	    text {
 		set d [string trim [$vw get 1.0 end] " \n"]
 	    }
+	    file=* {
+		set d [string trim [$vw.fn get]]
+	    }
 	    >* {
 		set d [$vw cget -text]
 	    }
@@ -350,7 +460,7 @@ proc DefData {kind frs types} {
 	    }
 	    default {
 		set d [$vw get]
-		if { ! $poss && "$d" != "" && [BadParam $TXT($kind) $t $d] } {
+		if { ! $poss && $d != "" && [BadParam $TXT($kind) $t $d] } {
 		    return $n
 		}
 	    }
@@ -364,25 +474,19 @@ proc DefData {kind frs types} {
 proc DefAction {act kind name oldname data} {
     # try to act on definition of object under $name (was $oldname)
     #  $act in {change, create, forget}
-    #  $kind in {datum, ellpsd}
+    #  $kind in {datum, ellpsd, backgrnd}
     #  $data is list with all the values except name (not used if $act==forget)
     # return 0 if action fails, and 1 otherwise
     # it is assumed that for each kind there a list of names of user-defined
     #  objects, data array indexed by the object name and a proc that is to
     #  be called upon a successful action
     # this proc is NOT checking if objects are in use when changing/forgetting
-    global MESS TXT GDATUM UDatums ELLPSDDEF UEllipsoids
+    global MESS TXT DefSpecs
 
-    switch $kind {
-	datum {
-	    set datarr GDATUM ; set ulist UDatums
-	    set savecomm SaveUserDatums
-	}
-	ellpsd {
-	    set datarr ELLPSDDEF ; set ulist UEllipsoids
-	    set savecomm SaveUserDatums
-	}
+    foreach v "datarr ulist savecomm" {
+	set $v $DefSpecs($kind,$v)
     }
+    global $datarr $ulist
     switch $act {
 	create {
 	    if { [lsearch -exact [array names $datarr] $name] != -1 } {
@@ -393,13 +497,16 @@ proc DefAction {act kind name oldname data} {
 	    set [set datarr]($name) $data
 	}
 	change {
-	    if { "$name" != "$oldname" } {
+	    if { $name != $oldname } {
 		if { [lsearch -exact [array names $datarr] $name] != -1 } {
 		    GMMessage $MESS(idinuse)
 		    return 0
 		}
 		set $ulist [lsort -dictionary \
-			[linsert [Delete [set $ulist] $oldname] 0 $name]]
+		       [linsert [Delete [set $ulist] $oldname] 0 $name]]
+		if { ! [catch {set p $DefSpecs($kind,chgcomm)}] } {
+		    $p $oldname $name
+		}
 		unset [set datarr]($oldname)
 	    }
 	    set [set datarr]($name) $data
@@ -408,6 +515,9 @@ proc DefAction {act kind name oldname data} {
 	    if { ! [GMConfirm [format $MESS(askforget) $TXT($kind)]] } {
 		return 0
 	    }
+	    if { ! [catch {set p $DefSpecs($kind,chgcomm)}] } {
+		$p $oldname ""
+	    }
 	    set $ulist [Delete [set $ulist] $oldname]
 	    unset [set datarr]($oldname)
 	}
