@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: files.tcl
-#  Last change:  20 March 2006
+#  Last change:  16 April 2007
 #
 # See also files_foreign.tcl
 #  and metadata.tcl for properties of known file formats
@@ -488,14 +488,17 @@ proc SaveTRComp {file w} {
     puts $file $MESS(TRcomp)
     set w $w.fr.fr3
     foreach n [$w.frbx.bxn get 0 end] \
-	    l [$w.frbx.bxl get 0 end] \
-	    k [$w.frbx.btl get 0 end] \
-	    t [$w.frbx.bdt get 0 end] \
+	    d [$w.frbx.bxd get 0 end] \
+	    la [$w.frbx.bxlat get 0 end] \
+	    lo [$w.frbx.bxlong get 0 end] \
 	    a [$w.frbx.bal get 0 end] \
+	    l [$w.frbx.bxl get 0 end] \
+	    tl [$w.frbx.btl get 0 end] \
+	    dt [$w.frbx.bdt get 0 end] \
 	    s [$w.frbx.bsp get 0 end] \
 	    b [$w.frbx.bbg get 0 end] {
 	if { [SlowOpAborted] } { return }
-	puts $file "$n\t$l\t$k\t$a\t$t\t$s\t$b"
+	puts $file "$n\t$d\t$la\t$lo\t$a\t$l\t$tl\t$dt\t$s\t$b"
     }
     foreach k "frt.td frt.tt frsp.max frsp.min" {
 	puts $file [$w.$k cget -text]
@@ -2582,6 +2585,74 @@ proc LoadMapInfo {path} {
     return $r
 }
 
+array set LSqIStates {
+    start     {format datum pformt}
+    start,nxt {point pformt datum}
+    pformt    {pformt}
+    pformt,nxt {datum}
+    datum     {datum}
+    datum,nxt {pformt}
+    point     {}
+}
+
+proc LoadLeastSquaresInfo {} {
+    # read geodetic and pixel coordinates of points for geo-referencing
+    #  a map image using a least squares fit
+    # file starts with optional datum and position format commands, or a
+    #  format command, (defaults are "WGS 84" and DDD), followed by a line
+    #  for each point with fields separated by tabulations containing the
+    #  position, in the selected/default datum and format, and the
+    #  pixel coordinates (x, y, with y increasing downwards)
+    # the total number of points must
+    # return 0 on failure, or a pair with list of latd,longd,datum, and list
+    #  of pairs with pixel coordinates (x, y, in Tcl sign convention with
+    #  origin at upper left corner)
+    global LSqIStates LFileLNo LFileEOF LFileBuffFull LFileDatum PosnLength \
+	TXT MESS MAPTRANSFNPTS
+
+    if { [set f [GMOpenFile $TXT(loadfrm) MapBkInfo r]] == ".." } {
+	return 0
+    }
+    set LFileLNo($f) 0 ; set LFileBuffFull($f) 0 ; set LFileEOF($f) 0
+    # defaults
+    set LFileDatum($f) "WGS 84" ; set LFilePFrmt($f) DDD
+    set state start ; set err 0
+    set llds "" ; set xys ""
+    while { ! $LFileEOF($f) } {
+	if { [set line [CleanLine [ReadFile $f] 0]] == "" } { continue }
+	if { [string first ! $line] == 0 } {
+	    set c [ProcCommand $line $f]
+	    if { [set ix [lsearch -exact $LSqIStates($state) $c]] == -1 } {
+		incr err ; break
+	    }
+	    set state [lindex $LSqIStates($state,nxt) $ix]
+	} else {
+	    set state point
+	    set fs [split $line \t]
+	    if { [set p [ReadPosn $fs 0 $f]] == -1 } { incr err ; break }
+	    set x [lindex $fs $PosnLength]
+	    set y [lindex $fs [expr $PosnLength+1]]
+	    if { ! [CheckSignedFloat GMMessage $x] || \
+		     ! [CheckSignedFloat GMMessage $y] } {
+		incr err ; break
+	    }
+	    lappend llds [list [lindex $p 0] [lindex $p 1] $LFileDatum($f)]
+	    lappend xys [list $x $y]
+	}
+    }
+    close $f
+    if { $err } {
+	GMMessage "$MESS(loaderr) $LFileLNo($f)"
+	return 0
+    }
+    if { [regexp {^>=([0-9]+)$} $MAPTRANSFNPTS(LeastSquares) x n] && \
+	     [lindex $llds [expr $n-1]] == "" } {
+	GMMessage [format $MESS(needNpoints) $n]
+	return 0
+    }
+    return [list $llds $xys]
+}
+
 ## BSB contribution
 proc LoadAutoMapInfo {} {
     # Load index of mapfiles and bounds
