@@ -1307,11 +1307,12 @@ proc PrepData {} {
 		set p [ToDatum [lindex $p 0] [lindex $p 1] \
 			   $WPDatum($ix) "WGS 84"]
 	    }
+	    set alt [lindex $WPAlt($ix) 0]
 	    return [list $PDTYPE($PkOutState) \
 			[PrepWPData $PkOutState \
 			     $WPName($ix) $WPCommt($ix) $p \
 			     $WPDate($ix) $WPSymbol($ix) \
-			     $WPDispOpt($ix) $WPAlt($ix) $WPHidden($ix)]]
+			     $WPDispOpt($ix) $alt $WPHidden($ix)]]
 	}
 	D200 -  D201 -  D202 {
 	    # RT header
@@ -1331,7 +1332,7 @@ proc PrepData {} {
 	    if { $PkOutSaved != "" } {
 		# $PkOutSaved is list of TPs
 		set p [lindex $PkOutSaved 0]
-		set alt [lindex $p $DataIndex(TPalt)]
+		set alt [lindex [lindex $p $DataIndex(TPalt)] 0]
 		set depth [lindex $p $DataIndex(TPdepth)]
 		set PkOutSaved [lreplace $PkOutSaved 0 0]
 		set new 0
