@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: garmin_protocols.tcl
-#  Last change:  10 April 2006
+#  Last change:  31 May 2007
 #
 
 # description of Garmin protocols based on Garmin GPS Interface
@@ -26,6 +26,7 @@
 #   Garmin GPS Interface Specification Version 1 Rev. 3
 #   Garmin GPS Interface Specification Version 1 Rev. A
 #   Garmin GPS Interface Specification Version 1 Rev. B
+#   Garmin GPS Interface Specification Version 1 Rev. C
 
 # Garmin USB protocol is implemented assuming that the USB port is driven by
 #  the garmin_gps Linux kernel driver by Hermann Kneissel and part of the
@@ -67,6 +68,7 @@ array set PROTCAT {
     A650  FlightBook
     A700  Posn
     A800  PVT
+    A801  nospec
     A802  nospec
     A900  nospec
     A901  nospec
@@ -76,7 +78,23 @@ array set PROTCAT {
     A905  nospec
     A906  LAP
     A907  nospec
+    A908  nospec
+    A913  nospec
     A914  nospec
+    A916  nospec
+    A917  nospec
+    A918  nospec
+    A1000 RUN
+    A1002 WKO
+    A1003 WKOOcc
+    A1004 FUP
+    A1005 WKOLimits
+    A1006 nospec
+    A1007 nospec
+    A1008 nospec
+    A1009 CRSLimits
+    A1010 nospec
+    A1011 nospec
 }
 
    # protocol requirements
@@ -102,6 +120,7 @@ array set PROTREQ {
     A650  ""
     A700  PosnData
     A800  PVTData
+    A801  D?
     A802  D?
     A900  ""
     A901  D?
@@ -111,7 +130,23 @@ array set PROTREQ {
     A905  D?
     A906  LAPData
     A907  {D? D? D? D?}
+    A908  D?
+    A913  ""
     A914  ""
+    A916  ""
+    A917  D?
+    A918  D?
+    A1000 RUNData
+    A1002 WKOData
+    A1003 WKOOcc
+    A1004 FUPData
+    A1005 WKOLimits
+    A1006 CRSData
+    A1007 D?
+    A1008 D?
+    A1009 CRSLimits
+    A1010 ""
+    A1011 ""
 }
 
    # Garmin USB Protocol
@@ -127,24 +162,40 @@ array set GUSB {
     PRIV,PID,SET,DEF,MODE 6
     MODE,NATIVE          0
     MODE,GARMIN,SERIAL   1
+    PID,TRANSPORT,DATA,AVAILABLE 2
     PID,TRANSPORT,START,SESSION,REQ 5
     PID,TRANSPORT,START,SESSION,RESP 6
 }
 
    # Basic Link Protocol
-set PID(ACK) 6  ; set RPID(6) ACK
-set PID(NAK) 21 ; set RPID(21) NAK
-set PID(PArray) 253 ; set RPID(253) PArray
-set PID(PrdReq) 254 ; set RPID(254) PrdReq
-set PID(PrdData) 255 ; set RPID(255) PrdData
-set PID(PrdExtData) 248 ; set RPID(248) PrdExtData
+array set PID {
+    ACK           6
+    NAK          21
+    PrdExtData  248
+    PArray      253
+    PrdReq      254
+    PrdData     255
+}
+
+array set RPID {
+    6           ACK
+    21          NAK
+    248  PrdExtData
+    253      PArray
+    254      PrdReq
+    255     PrdData
+}
 
    # definitions of Link product specific protocols
 set PSPDEF(L001) {{CmdData 10} {XfrCmpl 12} {DtTmData 14} {PosnData 17}
                   {PrxWPData 19} {Records 27} {RTHeader 29} {RTWPData 30}
                   {ALData 31} {TRData 34} {WPData 35} {PVTData 51}
                   {RTLinkData 98} {TRHeader 99} {FlightBook_Record 134}
-                  {LAPData 149} {WPCatData 152}}
+                  {LAPData 149} {WPCatData 152} {RUNData 990} {WKOData 991}
+                  {WKOOcc 992} {FUPData 993} {WKOLimits 994} {CRSData 1061}
+                  {CRSLap 1062} {CRSPoint 1063} {CRSTRHeader 1064}
+                  {CRSTRData 1065} {CRSLimits 1066}}
+
 set PSPDEF(L002) {{ALData 4} {CmdData 11} {XfrCmpl 12} {DtTmData 20}
                   {PosnData 24} {PrxWPData 27} {Records 35} {RTHeader 37}
                   {RTWPData 39} {WPData 43}}
@@ -155,7 +206,10 @@ set PSPDEF(L002) {{ALData 4} {CmdData 11} {XfrCmpl 12} {DtTmData 20}
     # definition of product specific device commands
 set PSCMDDEF(A010) {{Abort 0} {XfrAL 1} {XfrPosnData 2} {XfrPrx 3} {XfrRT 4}
                   {XfrDtTmData 5} {XfrTR 6} {XfrWP 7} {TurnOff 8} {StartPVT 49}
-                  {StopPVT 50} {XfrFlightBook 92} {XfrLAP 117} {XfrWPCat 121}}
+                  {StopPVT 50} {XfrFlightBook 92} {XfrLAP 117} {XfrWPCat 121}
+                  {XfrRUN 450} {XfrWKO 451} {XfrWKOOcc 452} {XfrFUP 453}
+                  {XfrWKOLimits 454} {XfrCRS 561} {XfrCRSLap 562}
+                  {XfrCRSPoint 563} {XfrCRSTrack 564} {XfrCRSLimits 565}}
 set PSCMDDEF(A011) {{Abort 0} {XfrAL 1} {XfrRT 8} {XfrPrx 17} {XfrDtTmData 20}
                   {XfrWP 21} {TurnOff 26}}
 
@@ -181,6 +235,13 @@ set PDTYPE(NAK) union=int,byte
 # set PDTYPE(ACK) union=byte,int
 # set PDTYPE(NAK) union=byte,int
 
+    # protocols for which no output is available (as in the Garmin specs,
+    #  or because of no support or no complete specification available)
+    # list of pairs with item type and either empty (for any) or list of
+    #  pairs with protocol type and protocol identifier
+
+set PROTNOOUTPUT {{TR {{TR A302} {TRData D304}}} {Prx {}} {AL {}}}
+
 array set PDTYPE {
     PArray  starray=char,word
     PrdReq  ignored
@@ -221,6 +282,8 @@ array set PDTYPE {
     D300  {semicircle longword boolean}
     D301  {semicircle longword float float boolean}
     D302  {semicircle longword float float float boolean}
+    D303  {semicircle longword float byte}
+    D304  {semicircle longword float float byte byte boolean}
     D310  {boolean byte string}
     D311  word
     D312  {boolean byte string}
@@ -242,9 +305,28 @@ array set PDTYPE {
     D700  radian
     D800  {float float float float int double radian float float float float
            int long}
+    D801  nospec
     D802  nospec
+    D900  nospec
     D901  nospec
     D906  {longword longword float semicircle semicircle word byte byte}
+    D907  nospec
+    D908  nospec
+    D909  nospec
+    D910  nospec
+    D911  nospec
+    D917  nospec
+    D918  nospec
+    D1003 nospec
+    D1004 nospec
+    D1005 nospec
+    D1006 nospec
+    D1007 nospec
+    D1008 nospec
+    D1009 nospec
+    D1011 nospec
+    D1012 nospec
+    D1013 nospec
 }
 
 ## RT/TR Header protocols with a numeric identifier
@@ -337,15 +419,25 @@ array set PRODID {
     "Geko 201" 248
     "Geko 301" 256
     "Rino 130" 264
+    "GPS 18USB" 273
     "Forerunner" 282
     "Forerunner 301" 283
     "GPSmap 276C" 285
     "GPS 60" 289
     "GPSMap60C" 291
+    "GPSMap60CSX" 292
+    "GPSMap76CSX" 292
+    "GPSMap60" 308
     "ForeTrex" 314
     "eTrex Vista C" 315
     "eTrex Legend C" 315
     "eTrex Venture" 419
+    "eTrex Legend Cx" 421
+    "EDGE205" 450
+    "EDGE305" 450
+    "Forerunner 205" 484
+    "Forerunner 305" 484
+    "StreetPilot i2" 532
 }
 
     # product names and how to obain its product specific protocols
@@ -435,17 +527,24 @@ array set PNAME {
     248  "Geko 201"
     256  "Geko 301"
     264  "Rino 130"
+    273  "GPS 18USB"
     282  "Forerunner"
-    282  "Forerunner 301"
+    283  "Forerunner 301"
     285  "GPSmap 276C"
     289  "GPS 60"
     291  "GPSMap60C"
+    292  "GPSMap60CSX/76CSX"
     295  "eTrex Summit"
+    308  "GPSMap60"
     314  "ForeTrex"
     315  "eTrex Vista/Legend C"
     411  "eTrex Legend"
     419  "eTrex Venture"
     420  "eTrex Vista"
+    421  "eTrex Legend Cx"
+    450  "EDGE205/305"
+    484  "Forerunner 205/305"
+    532  "StreetPilot i2"
 }
 
 # the descriptions below are valid for all versions except:
@@ -459,6 +558,8 @@ array set PNAME {
 #  111+ version >= 2.7
 #  285 version < 3.0 (just a guess)
 #  285+ version >= 3.0
+#  484 version < 2.6 (just a guess)
+#  484+ version >= 2.6
 
       # the pairs in the following entries have a comparison to be made
       #  to the receiver version and the identifier to be used if the
@@ -469,6 +570,8 @@ array set PRTCLVERSION {
     77  {{{>= 361} 77+++} {{>= 350} 77++} {{>= 301} 77+}}
     111 {{{>= 270} 111+}}
     285 {{{>= 300} 285+}}
+    289 {{{>= 240} 289+}}
+    484 {{{>= 260} 484+}}
 }
 
 array set PRTCLDEF {
@@ -549,18 +652,27 @@ array set PRTCLDEF {
     248  array
     256  diff
     264  diff
+    273  array
     282  array
     283  array
     285  see=291
     285+ array
     289  see=315
+    289+ diff
     291  array
+    292  array
     295  see=248
+    308  see=289+
     314  array
     315  array
     411  diff
     419  array
     420  see=411
+    421  array
+    450  array
+    484  diff
+    484+ diff
+    532  array
 }
 
 ### there is no information on which receivers use the PVT protocol
@@ -780,6 +892,22 @@ array set PSPID {
     248,PVT A800
     248,PVTData D800
 
+    273,Link L001
+    273,DevCmd A010
+    273,WP N/A
+    273,RT N/A
+    273,TR N/A
+    273,Prx  N/A
+    273,AL A500
+    273,ALData D501
+    273,DtTm A600
+    273,DtTmData D600
+    273,Posn A700
+    273,PosnData D700
+    273,PVT A800
+    273,PVTData D800
+    273,nospec A601+D601+A801+D801+A802+D802+A902+A903
+
     282,Link L001
     282,DevCmd A010
     282,WP A100
@@ -874,6 +1002,29 @@ array set PSPID {
     291,PVTData D800
     291,nospec A601+D601+A900+A902+A903+A904
 
+    292,Link L001
+    292,DevCmd A010
+    292,WP A100
+    292,WPData D110
+    292,RT A201
+    292,RTHeader D202
+    292,RTWPData D110
+    292,RTLinkData D210
+    292,TR A301
+    292,TRHeader D312
+    292,TRData D302
+    292,Prx A400
+    292,PrxWPData D110
+    292,AL A500
+    292,ALData D501
+    292,DtTm A600
+    292,DtTmData D600
+    292,Posn A700
+    292,PosnData D700
+    292,PVT A800
+    292,PVTData D800
+    292,nospec A601+D601+A801+D801+A900+A902+A903+A904+A907+D907+D908+D909+D910+A908+D911+A914+A916+A917+D917+A918+D918+A1010+A1011
+
     314,Link L001
     314,DevCmd A010
     314,WP A100
@@ -939,9 +1090,74 @@ array set PSPID {
     419,PosnData D700
     419,PVT A800
     419,PVTData D800
+    419,nospec A900,A902,A903
+
+    421,Link L001
+    421,DevCmd A010
+    421,WP A100
+    421,WPData D110
+    421,RT A201
+    421,RTHeader D202
+    421,RTWPData D110
+    421,RTLinkData D210
+    421,TR A301
+    421,TRHeader D312
+    421,TRData D302
+    421,Prx A400
+    421,PrxWPData D110
+    421,AL A500
+    421,ALData D501
+    421,DtTm A600
+    421,DtTmData D600
+    421,Posn A700
+    421,PosnData D700
+    421,PVT A800
+    421,PVTData D800
+    421,nospec A601+D601+A900+A902+A903+A904+A907+D907+D908+D909+D910+A908+D911+A914+A916+A917+D917+A918+D918
+
+    450,Link L001
+    450,DevCmd A010
+    450,WP A100
+    450,WPData D110
+    450,RT A201
+    450,RTHeader D202
+    450,RTWPData D110
+    450,RTLinkData D210
+    450,TR A302
+    450,TRHeader D311
+    450,TRData D304
+    450,Prx N/A
+    450,AL A500
+    450,ALData D501
+    450,DtTm A600
+    450,DtTmData D600
+    450,Posn A700
+    450,PosnData D700
+    450,PVT A800
+    450,PVTData D800
+    450,nospec A601+D601+A801+D801+A902+A903+A907+D907+D908+D909+D910+A918+D918+A1000+D1009+A906+D1011+A1002+D1008+A1003+D1003+A1004+D1004+A1005+D1005+A1006+D1006+A1007+D1007+A1008+D1012+A1009+D1013
+
+    532,Link L001
+    532,DevCmd A010
+    532,WP A100
+    532,WPData D110
+    532,RT N/A
+    532,TR N/A
+    532,Prx N/A
+    532,AL A500
+    532,ALData D501
+    532,DtTm A600
+    532,DtTmData: D600
+    532,Posn A700
+    532,PosnData D700
+    532,PVT A800
+    532,PVTData D800
+    532,nospec A601+D601+A802+D802+A900+A902+A903+A904+A905+D900+A907+D907+D908+D909+D910+A908+D911+A913+A916+A917+D917+A918+D918
 }
 
           # must not differ on the Link and DevCmd protocols
+          # cannot refer to unknown/unsupported protocols except in
+          #  list of "nospec" protocols
 array set PSDIFF {
      7  {59 {RTHeader D200} {TR N/A}}
     13  {59 {RTHeader D200} {Prx A400} {PrxData D400}}
@@ -968,7 +1184,10 @@ array set PSDIFF {
    197  {177 {Prx N/A}}
    256  {248 {nospec A802}}
    264  {291 {nospec T001+A802+D802+A900+A902+A903}}
+   289+ {289 {nospec A601+D601+A900+A902+A903+A904+A907+D908+D909+D910+A914+A1010+A1011}}
    411  {419 {nospec A802+D802+A900+A902+A903}}
+   484  {450 {AL N/A} {nospec A601+D601+A902+A903+A907+D908+D908+D909+D910+A1000+D1009+A906+D1011+A1000+D1009+A1002+D1008+A1003+D1003+A1004+D1004+A1005+D1005+A1006+D1006+A1007+D1007+A1008+D1012+A1009+D1013}}
+   484+ {450 {nospec A601+D601+A801+D801+A902+A903+A907+D908+D908+D909+D910+A918+D918+A1000+D1009+A906+D1015+A1002+D1008+A1003+D1003+A1004+D1004+A1005+D1005+A1006+D1006+A1007+D1007+A1008+D1012+A1009+D1013+A1013+D1014}}
 }
 
 ## descriptions, positions in Garmin structures of GPSMan data
@@ -1025,13 +1244,13 @@ array set DATAFOR {
     D155,ps {0 1 3 8 12 13}
 
     D310,ns {Colour Name}
-    D310,ps {0 2}
+    D310,ps {1 2}
 
     D311,ns Name
     D311,ps 0
 
     D312,ns {Colour Name}
-    D312,ps {0 2}
+    D312,ps {1 2}
 
     D500,ns {week datatime clockc1 clockc2 ecc sqrta mnanom argprgee
              rightasc rtrightasc inclin}
@@ -1058,11 +1277,12 @@ array set DATAFOR {
 
 array set GARMINCOLOURS {
     D310 {black darkred darkgreen darkyellow darkblue darkmagenta darkcyan
-          lightgray darkgray red green yellow blue magenta cyan white}
+          lightgray darkgray red green yellow blue magenta cyan white
+          default}
     D310,dvar DEFTTRCOLOUR
     D312 {black darkred darkgreen darkyellow darkblue darkmagenta darkcyan
           lightgray darkgray red green yellow blue magenta cyan white
-	  transparent}
+	  transparent default}
     D312,dvar DEFTTRCOLOUR
 }
 
@@ -1225,14 +1445,14 @@ array set HIDDENFOR {
     D310,ns {display}
     D310,ts {boolean}
     D310,vs {0}
-    D310,ps {1}
+    D310,ps {0}
     D310,cs {}
     D310,fm {{display enumd {no  yes}}}
 
     D312,ns {display}
     D312,ts {boolean}
     D312,vs {0}
-    D312,ps {1}
+    D312,ps {0}
     D312,cs {}
     D312,fm {{display enumd {no  yes}}}
 }
