@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: gendials.tcl
-#  Last change:  6 April 2006
+#  Last change:  16 April 2007
 #
 # Includes contributions by
 #  - Brian Baulch (baulchb@onthenet.com.au) marked "BSB contribution"
@@ -134,9 +134,13 @@ proc GMSelect {mess blist vlist} {
     label .messres.fr.title -text "???" -relief sunken
     label .messres.fr.text -text $mess
     frame .messres.fr.frsel
+    set max 4 ; set c 0 ; set r 0
     foreach e $blist v $vlist {
 	button .messres.fr.frsel.b$e -text $e -command "set GMResSel $v"
-	pack .messres.fr.frsel.b$e -side left
+	grid .messres.fr.frsel.b$e -column $c -row $r -sticky ew
+	if { [incr c] >= $max } {
+	    set c 0 ; incr r
+	}
     }
     pack .messres.fr -side top
     pack .messres.fr.title .messres.fr.text .messres.fr.frsel -side top -pady 5
