@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: langdeutsch.tcl
-#  Last change:  8 May 2006
+#  Last change:  30 April 2007
 # German language file by Andreas Lange (Andreas.C.Lange@GMX.de)
 #  last input from him inserted: 31 January 2002
 #
@@ -52,7 +52,7 @@ set ARUNIT $DLUNIT($DISTUNIT,area)
 set ALUNIT $DLUNIT($ALTUNIT,dist)
 
 set MESS(RTcomp) "#\tWP\t\t$DTUNIT\tGrad\t$ALUNIT"
-set MESS(TRcomp) "#\t$DTUNIT\t$DTUNIT\t$ALUNIT\th:m:s\t$SPUNIT\tGrad"
+set MESS(TRcomp) "#\t\t\t\t$ALUNIT\t$DTUNIT\t$DTUNIT\th:m:s\t$SPUNIT\tdeg"
 set MESS(WPNearest) "WP\t\t$DTUNIT\tGrad"
 
 ## comments to Mess
@@ -70,7 +70,7 @@ array set MESS {
     namelgth   "Der Name darf %d Zeichen nicht berschreiten"
     cmmtlgth   "Der Kommentar darf %d Zeichen nicht berschreiten"
     namevoid   "Der Name darf nicht leer sein"
-    baddate   "Ungltiges Datum"
+    baddate   "Ungltiges Datum; Zeile"
     badhdg   "Ungltige Richtungsangabe %s: mu entweder %s oder +/- sein"
     badcoord   "Ungltige Koordinaten oder falsches Format: \"%s\" mu sein %s"
     outofrng   "Zahl auerhalb des zulssigen Bereiches"
@@ -293,10 +293,12 @@ array set MESS {
     cantwrtmbaks "Kann die nutzerdefinierte Hintergrund-Datei nicht schreiben"
     abbrevhasspaces "Abkrzungen drfen keine Leerzeichen haben"
     getAL "Empfangen des Almanachs"
+    needNpoints "Mindestens %s Punkten mu ausgewhlt werden!"
 }
 
 set TXT(RTcompflds) "# WP {$DTUNIT} Grad {$ALUNIT} Etappe Bezeichnung"
-set TXT(TRcompflds) "TP {$DTUNIT} {$DTUNIT} {$ALUNIT} h:m:s {$SPUNIT} Grad"
+set TXT(TRcompflds) \
+    "TP {} {} {} {$ALUNIT} {$DTUNIT} {$DTUNIT} h:m:s {$SPUNIT} Grad"
 set TXT(starttoend) "Start: bis Ende %s $DTUNIT;"
 set TXT(startmax) "max. %s $DTUNIT;"
 set TXT(WPnearflds) "WP {$DTUNIT} Grad"
@@ -1206,6 +1208,8 @@ array set TXT {
     alm_rtrightasc "Rektaszensionsgeschwindigkeit"
     alm_inclin "Inklination"
     alm_health "Gesundheit"
+    lstsqs  "Kleinsten Quadrate"
+    lstsqsfile "Kl. Quadrate Datei"
 }
 
     # the following definitions must be coherent with $TXT
