@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: langengl.tcl
-#  Last change:  6 April 2006
+#  Last change:  17 April 2007
 #
 # Includes contributions by
 #  - Brian Baulch (baulchb@onthenet.com.au)
@@ -51,7 +51,7 @@ set ARUNIT $DLUNIT($DISTUNIT,area)
 set ALUNIT $DLUNIT($ALTUNIT,dist)
 
 set MESS(RTcomp) "#\tWP\t\t$DTUNIT\tdeg\t$ALUNIT"
-set MESS(TRcomp) "#\t$DTUNIT\t$DTUNIT\t$ALUNIT\th:m:s\t$SPUNIT\tdeg"
+set MESS(TRcomp) "#\t\t\t\t$ALUNIT\t$DTUNIT\t$DTUNIT\th:m:s\t$SPUNIT\tdeg"
 set MESS(WPNearest) "WP\t\t$DTUNIT\tdeg"
 
 array set MESS {
@@ -61,7 +61,7 @@ array set MESS {
     namelgth   "Name must not exceed %d characters"
     cmmtlgth   "Comment must not exceed %d characters"
     namevoid   "Name cannot be void"
-    baddate   "Bad date"
+    baddate   "Bad date; line"
     badhdg   "Bad heading %s: must be %s or +/-"
     badcoord   "Bad coordinate range or format: \"%s\" must be %s"
     outofrng   "Number out of range"
@@ -292,10 +292,12 @@ array set MESS {
     cantwrtsymenu "Cannot write symbol menu file"
     cantwrtmbaks "Cannot write user map backgrounds file"
     abbrevhasspaces "Short name cannot have blanks"
+    needNpoints "You must give at least %s points!"
 }
 
 set TXT(RTcompflds) "# WP {$DTUNIT} deg {$ALUNIT} stage label"
-set TXT(TRcompflds) "TP {$DTUNIT} {$DTUNIT} {$ALUNIT} h:m:s {$SPUNIT} deg"
+set TXT(TRcompflds) \
+    "TP {} {} {} {$ALUNIT} {$DTUNIT} {$DTUNIT} h:m:s {$SPUNIT} deg"
 set TXT(starttoend) "Start: to end %s $DTUNIT;"
 set TXT(startmax) "max %s $DTUNIT;"
 set TXT(WPnearflds) "WP {$DTUNIT} deg"
@@ -1285,6 +1287,8 @@ array set TXT {
     alm_rtrightasc "rate r ascension"
     alm_inclin "inclination"
     alm_health "health"
+    lstsqs  "Least Squares"
+    lstsqsfile "Least Squares file"
 }
 
     # the following definitions must be coherent with $TXT
