@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 #      along with this program.
 #
 #  File: langes.tcl
-#  Last change:  12 April 2006
+#  Last change:  21 April 2007
 #
 # Spanish language file by Alberto Morales (amd77@gulic.org)
-#  last input from him: 8 April 2006
+#  last input from him: 21 April 2007
 #
 
 # file langengl.tcl is consulted first; no need for duplicating entries here
@@ -50,7 +50,7 @@ set ARUNIT $DLUNIT($DISTUNIT,area)
 set ARUNIT $DLUNIT($DISTUNIT,area)
 
 set MESS(RTcomp) "#\tPT\t\t$DTUNIT\tgrados\t$ALUNIT"
-set MESS(TRcomp) "#\t$DTUNIT\t$DTUNIT\t$ALUNIT\th:m:s\t$SPUNIT\tgraus"
+set MESS(TRcomp) "#\t\t\t\t$ALUNIT\t$DTUNIT\t$DTUNIT\th:m:s\t$SPUNIT\tgrados"
 set MESS(WPNearest) "PT\t\t$DTUNIT\tgrados"
 
 array set MESS {
@@ -60,7 +60,7 @@ array set MESS {
     namelgth   "Nombre no puede tener ms de %d caracteres"
     cmmtlgth   "Comentario no puede tener ms de %d caracteres"
     namevoid   "Nombre no puede ser vaco"
-    baddate   "Fecha incorrecta"
+    baddate   "Fecha incorrecta; linea"
     badhdg   "Direccin incorrecta %s: debe ser %s o +/-"
     badcoord "Coordenada con valor o formato incorrecto: \"%s\" debe ser %s"
     outofrng   "Nmero fuera de lmites"
@@ -283,10 +283,12 @@ array set MESS {
     cantwrtmbaks "No puedo escribir fichero de mapas de fondo de usuario"
     abbrevhasspaces "El nombre corto no puede tener espacios"
     getAL   "Obteniendo almanaque"
+    needNpoints "Debes dar al menos %s puntos!"
 }
 
 set TXT(RTcompflds) "# PT {$DTUNIT} grados {$ALUNIT} etapa etiq"
-set TXT(TRcompflds) "PTR {$DTUNIT} {$DTUNIT} {$ALUNIT} h:m:s {$SPUNIT} grados"
+set TXT(TRcompflds) \
+    "PTS {} {} {} {$ALUNIT} {$DTUNIT} {$DTUNIT} h:m:s {$SPUNIT} grados"
 set TXT(starttoend) "Inicio: hasta fin %s $DTUNIT;"
 set TXT(startmax) "mx %s $DTUNIT;"
 set TXT(WPnearflds) "PT {$DTUNIT} grados"
@@ -1225,33 +1227,35 @@ array set TXT {
     duration "Duracin"
     calrs "Caloras"
     getLAP  "Obteniendo Vueltas"
-   fromfile "desde fichero"
-   fromdef "desde definicin"
-   mbaktoload "Fondo para cargar"
-   none "ninguno"
-   opt_Fonts "Tamaos de fuentes"
-   optFONTSIZE "Tamao fuente por defecto"
-   optFIXEDFONTSIZE "Tamao fuente fija"
-   optICONSIZE "Tamao de iconos"
-   LP   PL
-   garmin_usb "Garmin USB"
-   optTRNUMBERINTVL "Mostrar en puntos trayecto en mapa"
-   optTRINFO "Informacin puntos trayecto para mostrar"
-   nameInfo "Informacin"
-   nameAL "Almanaque"
-   alm_svid "id satlite"
-   alm_week "semana"
-   alm_datatime "tiempo ref datos"
-   alm_clockc1 "cpef corr reloj s"
-   alm_clockc2 "coef corr reloj s/s"
-   alm_ecc "excentricidad"
-   alm_sqrta "raiz cuadr a"
-   alm_mnanom "anomala promedio"
-   alm_argprgee "arg perigeo"
-   alm_rightasc "ascensin recta"
-   alm_rtrightasc "veloc ascensin r"
-   alm_inclin "inclinacin"
-   alm_health "salud"
+    fromfile "desde fichero"
+    fromdef "desde definicin"
+    mbaktoload "Fondo para cargar"
+    none "ninguno"
+    opt_Fonts "Tamaos de fuentes"
+    optFONTSIZE "Tamao fuente por defecto"
+    optFIXEDFONTSIZE "Tamao fuente fija"
+    optICONSIZE "Tamao de iconos"
+    LP   PL
+    garmin_usb "Garmin USB"
+    optTRNUMBERINTVL "Mostrar en puntos trayecto en mapa"
+    optTRINFO "Informacin puntos trayecto para mostrar"
+    nameInfo "Informacin"
+    nameAL "Almanaque"
+    alm_svid "id satlite"
+    alm_week "semana"
+    alm_datatime "tiempo ref datos"
+    alm_clockc1 "cpef corr reloj s"
+    alm_clockc2 "coef corr reloj s/s"
+    alm_ecc "excentricidad"
+    alm_sqrta "raiz cuadr a"
+    alm_mnanom "anomala promedio"
+    alm_argprgee "arg perigeo"
+    alm_rightasc "ascensin recta"
+    alm_rtrightasc "veloc ascensin r"
+    alm_inclin "inclinacin"
+    alm_health "salud"
+    lstsqs "Mnimos cuadrados"
+    lstsqsfile  "Fichero de Mnimos cuadrados"
 }
 
     # the following definitions must be coherent with $TXT
