@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,11 +17,11 @@
 #      along with this program.
 #
 #  File: langengl.tcl
-#  Last change:  3 May 2006
+#  Last change:  20 April 2007
 #
 # French language file by Valre Robin (valere.robin@wanadoo.fr)
 #  Last change by him:  11 November 2001
-#  last input from him inserted:  27 July 2004
+#  last input from him inserted:  20 April 2007
 #
 # Includes contributions by Jean H Theoret (ve2za@rac.ca)
 # Includes typo corrections by Laurent Bonnaud (bonnaud@lis.inpg.fr)
@@ -49,7 +49,7 @@ set ARUNIT $DLUNIT($DISTUNIT,area)
 set ALUNIT $DLUNIT($ALTUNIT,dist)
 
 set MESS(RTcomp) "#\tWP\t\t$DTUNIT\tdeg\t$ALUNIT"
-set MESS(TRcomp) "#\t$DTUNIT\t$DTUNIT\t$ALUNIT\th:m:s\t$SPUNIT\tdeg"
+set MESS(TRcomp) "#\t\t\t\t$ALUNIT\t$DTUNIT\t$DTUNIT\th:m:s\t$SPUNIT\tdeg"
 set MESS(WPNearest) "WP\t\t$DTUNIT\tdeg"
 
 array set MESS {
@@ -59,7 +59,7 @@ array set MESS {
     namelgth   "Le nom ne doit pas dpasser %d caractres"
     cmmtlgth   "=== Un commentaire ne peut dpasser %d caractres"
     namevoid   "Le nom ne peut tre vide"
-    baddate   "Date errone"
+    baddate   "Date errone; ligne"
     badhdg   "Mauvais en-tte %s: doit tre %s ou +/-"
     badcoord   "Mauvais format ou intervalle de coordonnes: \"%s\" doit tre %s"
     outofrng   "Nombre non valide"
@@ -282,10 +282,12 @@ array set MESS {
     cantwrtmbaks "Impossible d'criture le fichier de fond de carte"
     abbrevhasspaces "Nom court ne peut pas avoir des blancs"
     getAL  "Acquisition de l'almanach"
+    needNpoints "Merci de fournir au moins %s points!"
 }
 
 set TXT(RTcompflds) "# WP {$DTUNIT} deg {$ALUNIT} stage label"
-set TXT(TRcompflds) "TP {$DTUNIT} {$DTUNIT} {$ALUNIT} h:m:s {$SPUNIT} deg"
+set TXT(TRcompflds) \
+    "TP {} {} {} {$ALUNIT} {$DTUNIT} {$DTUNIT} h:m:s {$SPUNIT} deg"
 set TXT(starttoend) "Dpart: vers la fin %s $DTUNIT;"
 set TXT(startmax) "max %s $DTUNIT;"
 set TXT(WPnearflds) "WP {$DTUNIT} deg"
@@ -1095,7 +1097,7 @@ array set TXT {
     SYpin_green "Epingle, verte"
     SYpin_red "Epingle, rouge"
     changegroupsymbol "Changer le symbole"
-    ozimapfile "Ficher Ozi Map"
+    ozimapfile "Fichier Ozi Map"
     info "Information"
     climbrate "Vitesse verticale"
     optUSESLOWOPWINDOW "Fentre de contrle des oprations lentes"
@@ -1240,6 +1242,8 @@ array set TXT {
     alm_rtrightasc "taxe de ascension droite"
     alm_inclin "inclinaison"
     alm_health "sant"
+    lstsqs "Moindres carrs"
+    lstsqsfile "Fichier des moindres carrs"
 }
 
     # the following definitions must be coherent with $TXT
