@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,10 +17,10 @@
 #      along with this program.
 #
 #  File		: langid.tcl
-#  Last change	:  16 October 2004
+#  Last change	:  19 April 2007
 #
 # Indonesian language file by Tri Agus Prayitno (acuss@bk.or.id)
-#  last input from him inserted:  15 July 2004
+#  last input from him inserted:  19 April 2007
 #
 
 # file langengl.tcl is consulted first; no need for duplicating entries here
@@ -50,7 +50,7 @@ set ARUNIT $DLUNIT($DISTUNIT,area)
 set ALUNIT $DLUNIT($ALTUNIT,dist)
 
 set MESS(RTcomp) "#\tWP\t\t$DTUNIT\tdeg\t$ALUNIT"
-set MESS(TRcomp) "#\t$DTUNIT\t$DTUNIT\t$ALUNIT\th:m:s\t$SPUNIT\tdeg"
+set MESS(TRcomp) "#\t\t\t\t$ALUNIT\t$DTUNIT\t$DTUNIT\th:m:s\t$SPUNIT\tdeg"
 set MESS(WPNearest) "WP\t\t$DTUNIT\tdeg"
 
 array set MESS {
@@ -60,7 +60,7 @@ array set MESS {
     namelgth   "Nama tidak boleh lebih dari %d karakter"
     cmmtlgth   "Komentar tidak boleh lebih dari %d karakter"
     namevoid   "Nama harus diisi"
-    baddate   "Tanggal salah"
+    baddate   "Tanggal salah; baris"
     badhdg   "Heading salah %s: harus %s atau +/-"
     badcoord   "Rentang/format koordinat salah: \"%s\", harusnya %s"
     outofrng   "Jumlah di luar jangkauan"
@@ -273,10 +273,12 @@ array set MESS {
     RTnoWPname "Fitur WP dalam RT menurut nama saja sudah tidak ada"
     distlarge "Jarak terlalu besar!"
     timeconsmg "Operasi ini sangat lama: lanjutkan?"
+    needNpoints "Anda harus memasukkan minimal %s titik!"
 }
 
 set TXT(RTcompflds) "# WP {$DTUNIT} drj {$ALUNIT} tahap label"
-set TXT(TRcompflds) "TP {$DTUNIT} {$DTUNIT} {$ALUNIT} h:m:s {$SPUNIT} deg"
+set TXT(TRcompflds) \
+    "TP {} {} {} {$ALUNIT} {$DTUNIT} {$DTUNIT} h:m:s {$SPUNIT} deg"
 set TXT(starttoend) "Mulai: untuk mengakhiri %s $DTUNIT;"
 set TXT(startmax) "maks %s $DTUNIT;"
 set TXT(WPnearflds) "WP {$DTUNIT} drj"
@@ -1099,6 +1101,7 @@ array set TXT {
     optDEFTRECPROTOCOL "Protokol default"
     optLNSREACT "Reaksi LN thd mouse"
     syhlpbx "Gunakan klik-kanan untuk\nmengubah daftar elemen"
+    lstsqsfile "File Least Squares"
 }
 
     # the following definitions must be coherent with $TXT
