@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,9 +17,9 @@
 #      along with this program.
 #
 #  File: langit.tcl
-#  Last change:  12 April 2006
+#  Last change:  19 April 2007
 #  Italian language file by Alessandro Palmas (alpalmas@tin.it)
-#  last input from him inserted:  12 April 2006
+#  last input from him inserted:  19 April 2007
 #
 
 # file langengl.tcl is consulted first; no need for duplicating entries here
@@ -47,7 +47,7 @@ set SPUNIT $DLUNIT($DISTUNIT,speed)
 set ALUNIT $DLUNIT($ALTUNIT,dist)
 
 set MESS(RTcomp) "#\tWP\t\t$DTUNIT\tdeg\t$ALUNIT"
-set MESS(TRcomp) "#\t$DTUNIT\t$DTUNIT\t$ALUNIT\th:m:s\t$SPUNIT\tdeg"
+set MESS(TRcomp) "#\t\t\t\t$ALUNIT\t$DTUNIT\t$DTUNIT\th:m:s\t$SPUNIT\tdeg"
 set MESS(WPNearest) "WP\t\t$DTUNIT\tdeg"
 
 array set MESS {
@@ -57,7 +57,7 @@ array set MESS {
     namelgth   "I nomi non debbono superare i %d caratteri"
     cmmtlgth   "I commenti non debbono superare i %d caratteri"
     namevoid   "Il nome non pu essere lasciato in bianco"
-    baddate   "Data non corretta"
+    baddate   "Data non corretta; linea"
     badhdg   "inizio non corretto %s: deve essre %s o +/-"
     badcoord   "Valore o formato delle coordinate non corretto: \"%s\" deve essere %s"
     outofrng   "Il numero eccede i limiti"
@@ -281,10 +281,12 @@ array set MESS {
     cantwrtsymenu "Non posso scrivere il file del men dei simboli"
     cantwrtmbaks "Impossibile scrivere file mappe background"
     abbrevhasspaces "I nomi corti non debbono contenere spazi"
+    needNpoints "Fornire almeno %s punti"
 }
 
 set TXT(RTcompflds) "# WP {$DTUNIT} deg {$ALUNIT} stage label"
-set TXT(TRcompflds) "TP {$DTUNIT} {$DTUNIT} {$ALUNIT} h:m:s {$SPUNIT} deg"
+set TXT(TRcompflds) \
+    "TP {} {} {} {$ALUNIT} {$DTUNIT} {$DTUNIT} h:m:s {$SPUNIT} deg"
 set TXT(starttoend) "Start: to end %s $DTUNIT;"
 set TXT(startmax) "max %s $DTUNIT;"
 set TXT(WPnearflds) "WP {$DTUNIT} deg"
@@ -1185,6 +1187,8 @@ array set TXT {
     alm_rtrightasc "var. asc. retta"
     alm_inclin "inclinazione"
     alm_health "salute"
+    lstsqs "Minimi Quadrati"
+    lstsqsfile "file Min Quadrati"
 }
 
     # the following definitions must be coherent with $TXT
