@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,9 +17,9 @@
 #      along with this program.
 #
 #  File: langnl.tcl
-#  Last change:  26 April 2006
+#  Last change:  20 April 2007
 #  Dutch language file by Rob Buitenhuis (rob@buitenhs.demon.nl)
-#  last input from him inserted: 26 April 2006
+#  last input from him inserted: 20 April 2007
 #
 # 
 #  file langengl.tcl is consulted first; no need for duplicating entries here
@@ -49,7 +49,7 @@ set SPUNIT $DLUNIT($DISTUNIT,speed)
 set ALUNIT $DLUNIT($ALTUNIT,dist)
 
 set MESS(RTcomp) "#\tWP\t\t$DTUNIT\tgrd\t$ALUNIT"
-set MESS(TRcomp) "#\t$DTUNIT\t$DTUNIT\t$ALUNIT\th:m:s\t$SPUNIT\tgrd"
+set MESS(TRcomp) "#\t\t\t\t$ALUNIT\t$DTUNIT\t$DTUNIT\th:m:s\t$SPUNIT\tgrd"
 set MESS(WPNearest) "WP\t\t$DTUNIT\tgrd"
 
 ## comments to Mess
@@ -67,7 +67,7 @@ array set MESS {
     namelgth   "De naam mag de %d tekens niet overschrijden"
     cmmtlgth   "Het commentaar mag de %d tekens niet overschrijden"
     namevoid   "Het naamveld mag niet leeg zijn"
-    baddate    "Niet geldige datum"
+    baddate    "Niet geldige datum; regel"
     badhdg   "Niet geldige richting %s: moet of %s of +/- zijn"
     badcoord   "Niet geldige coordinaten of foutieve weergave: \"%s\" moet %s zijn"
     outofrng   "Getal buiten grenzen van toegestaan bereik"
@@ -290,10 +290,12 @@ array set MESS {
     cantwrtmbaks "Kan het achtergrondenbestand niet schrijven"
     abbrevhasspaces "Afgekorte naam mag geen spaties bevatten"
     getAL   "Verkrijgen almanak"
+    needNpoints "U moet tenminste %s punten opgeven!"
 }
 
 set TXT(RTcompflds) "# WP {$DTUNIT} graad {$ALUNIT} et. beschr."
-set TXT(TRcompflds) "TP {$DTUNIT} {$DTUNIT} {$ALUNIT} u:m:s {$SPUNIT} graad"
+set TXT(TRcompflds) \
+    "TP {} {} {} {$ALUNIT} {$DTUNIT} {$DTUNIT} u:m:s {$SPUNIT} graad"
 set TXT(starttoend) "Start: tot eind %s $DTUNIT;"
 set TXT(startmax) "max. %s $DTUNIT;"
 set TXT(WPnearflds) "WP {$DTUNIT} graad"
@@ -1218,6 +1220,8 @@ array set TXT {
     alm_rtrightasc "snelh r klimming"
     alm_inclin "inclinatie"
     alm_health "geldigheid"
+    lstsqs "Kl. kwadraten"
+    lstsqsfile "Kl. kwadr. bestand"
 }
 
     # the following definitions must be coherent with $TXT
