@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: langport.tcl
-#  Last change:  6 April 2006
+#  Last change:  16 April 2007
 #
 
 # file langengl.tcl is consulted first; no need for duplicating entries here
@@ -47,7 +47,7 @@ set ARUNIT $DLUNIT($DISTUNIT,area)
 set ALUNIT $DLUNIT($ALTUNIT,dist)
 
 set MESS(RTcomp) "#\tPT\t\t$DTUNIT\tgraus\t$ALUNIT"
-set MESS(TRcomp) "#\t$DTUNIT\t$DTUNIT\t$ALUNIT\th:m:s\t$SPUNIT\tgraus"
+set MESS(TRcomp) "#\t\t\t\t$ALUNIT\t$DTUNIT\t$DTUNIT\th:m:s\t$SPUNIT\tgraus"
 set MESS(WPNearest) "PT\t\t$DTUNIT\tgrau"
 
 array set MESS {
@@ -57,7 +57,7 @@ array set MESS {
     namelgth   "Nome no poder ter mais de %d caracteres"
     cmmtlgth   "Comentrio no poder ter mais de %d caracteres"
     namevoid   "Nome no pode ser vazio"
-    baddate   "Data errada"
+    baddate   "Data errada; linha"
     badhdg   "Direco errada %s: tem de ser %s ou +/-"
     badcoord "Coordenada com valor ou formato errado: \"%s\" tem de ser em %s"
     outofrng   "Nmero fora dos limites"
@@ -288,10 +288,12 @@ array set MESS {
     cantwrtsymenu "No posso escrever no ficheiro do menu de smbolos"
     cantwrtmbaks "No posso escrever no ficheiro de mapas"
     abbrevhasspaces "Abreviatura no pode ter espaos"
+    needNpoints "Precisa de pelo menos %s pontos!"
 }
 
 set TXT(RTcompflds) "# PT {$DTUNIT} grau {$ALUNIT} etapa etiq"
-set TXT(TRcompflds) "PTR {$DTUNIT} {$DTUNIT} {$ALUNIT} h:m:s {$SPUNIT} grau"
+set TXT(TRcompflds) \
+    "PTR {} {} {} {$ALUNIT} {$DTUNIT} {$DTUNIT} h:m:s {$SPUNIT} grau"
 set TXT(starttoend) "Incio: ao fim %s $DTUNIT;"
 set TXT(startmax) "no mx %s $DTUNIT;"
 set TXT(WPnearflds) "PT {$DTUNIT} grau"
@@ -1230,6 +1232,8 @@ array set TXT {
     alm_rtrightasc "taxa de asc recta"
     alm_inclin "inclinao"
     alm_health "sade"
+    lstsqs  "Min Quadrados"
+    lstsqsfile "Min Quad ficheiro"
 }
 
     # the following definitions must be coherent with $TXT
