@@ -16,39 +16,43 @@
 #      You should have received a copy of the GNU General Public License
 #      along with this program.
 #
-#  File: lowrance_nmea.tcl - copyright (c) 2000 Brian Baulch - baulchb@hotkey.net.au
-#  Last change:  26 Jan 2002
-#
+#  File: lowrance_nmea.tcl - copyright (c) 2000,2002 Brian Baulch - baulchb@hotkey.net.au
+#  Last change: 14 Apr 2002
+#  Procs WriteHeader1 & SaveTR1 are derived from procs WriteHeader and
+#    SaveTR - both copyright (c) 2000 Miguel Figueras.
 
-set LOGMENU {1s 2s 3s 4s 5s 10s 15s 20s 25s 30s 35s 40s 45s 50s 55s 1m 2m \
-		3m 4m 5m 10m 15m 20m 25m 30m}
-set LOGCMD {1 2 3 4 5 10 15 20 25 30 35 40 45 50 55 60 120 \
-		180 240 300 600 900 1200 1500 1800}
+set LOGMENU {1s 2s 3s 4s 5s 6s 10s 15s 20s 25s 30s 45s 60s}
+set LOGCMD {1 2 3 4 5 6 10 15 20 25 30 45 60}
+set CurrentPath ""
 set FirstFlag 1
 set SecondsInDay 86400
-set NMEATIMEOUT 2000
-set NMEATrackName NMEA1
+set NMEATIMEOUT 10000
+set NMEALastLogLength 60
 set NMEALogInterval 10
+set NMEATrackName "NMEA1"
+set TrackCountFile "track-count"
+# set NMEALogInterval 30
 set NMEALastTick 0
 array set TXT {
     NMEAauto "autoMapic plot"
-    NMEApreload "Preload plot"
+    NMEApreload "Manual plot"
     GMNmea "GPS Manager - NMEA"
     Gman "GPS Manager"
     start START
     Interval Interval
     NMEAtitle "NMEA logging/plotting"
-    Autoload "Load autoMAPic Image file"
-    NMEASet "NMEA Settings"
-    Newtrack "Enter name for new Track"
+    Autoload "autoMapic image file"
+    NMEASet "Real-time log settings"
+    Newtrack "Manual plot filename"
     Nmealogpgr "NMEA logging in progress"
     stop STOP
 }
+set ListExists 1
 set MapName ""
 set MESS(Noimg) "THERE IS NO MAP IMAGE\nAVAILABLE FOR THIS POSITION"
 set NMEANoImage 0
-set NMEADatum "Australian Geod `66"
-# set NMEADatum "WGS 84"
+set NMEANoImageTrackFamily "Blank"
+set NMEADatum "WGS 84"
 set NMEAReadDone 1
 # set NMEABaud 19200
 set NMEABaud 4800
@@ -57,19 +61,19 @@ set NMEABaud 4800
 
 proc StartNMEALog {type} {
     # Type 0 = autoMapic loading
-    # Type 1 = Preloaded image
+    # Type 1 = Manual loading
 
-    global HaltNMEA NotLogging SERIALLOG NMEALogInterval NMEATrackName \
-		NMEABulkData Rollover Datum Type
+    global HaltNMEA NotLogging SERIALLOG NMEALogInterval \
+	NMEABulkData Rollover Type
 
     set Type $type
     set Rollover 0
     set NMEABulkData ""
     if {! $Type} { raise . }
-    if { [NMEAInProgrWindow] } {
-	 return } 
+    if {$Type} {destroy .nmea}
     set HaltNMEA 0
     OpenNMEAPort
+    if {! $Type} {SetTrackCountChannel}    
     NMEAInterface
     return
 }
@@ -117,7 +121,7 @@ proc NMEATimeOut {} {
 proc NMEAInterface {} {
 
 
-    global Id NotLogging NMEAReadState NMEATIMEOUT SRLFILE
+    global Id NotLogging NMEAReadState NMEATIMEOUT
 
     set NMEAReadState ""
     set Id [after 0 ReadNMEAInputLine]
@@ -132,7 +136,7 @@ proc ReadNMEAInputLine {} {
 # The "fileevents" handler.
 
     global NMEAFirstLine NMEAInputLine NMEAReadState NMEAReadDone \
-		SRLFILE
+	SRLFILE
 
     set ch [read $SRLFILE 1]
     if {$ch == ""} {return}
@@ -184,6 +188,7 @@ proc ProcessNMEAInputList {} {
 
     set linelist $NMEAInputList
     set data [CreateNMEALists $linelist]
+    if {$data == ""} {return}
     if {$NMEASecondLine} {
 	set NMEASecondLine 0
 	set lastick [NMEATimeSeconds $linelist] 
@@ -212,6 +217,7 @@ proc ProcessNMEAInputList {} {
 proc CreateNMEALists {linelist} {
 
     set lat [lindex $linelist 2]
+    if {$lat == ""} {return ""}
     set latsign [lindex $linelist 3]
     set long [lindex $linelist 4]
     set longsign [lindex $linelist 5]
@@ -274,7 +280,7 @@ proc ConvNMEAToDegrees {val} {
 	
 proc NMEAFormatInData {posn} {
 
-    global NMEATrackName NMEABulkData LogFile
+    global NMEATrackName NMEABulkData
 
     set secs [clock seconds]
     set time [TimeStamp $secs]
@@ -286,19 +292,18 @@ proc NMEAFormatInData {posn} {
 }  
 
 
-proc NMEAInDataTR {} {
+proc NMEAInDataTR {id bulkdata} {
     # add TRs data from NMEA receiver to database
 
-    global Index Number GetDispl NMEABulkData
+    global DataDefault NMEADatum Index Number GetDispl
 
-    set p [lindex NMEABulkData 0]
+    set DataDefault(TR) [list "" "" $NMEADatum "" "" 0]
     set tps ""
-    set id [lindex $p 0]
     set ix [IndexNamed TR $id]
     if {$ix == -1} then {
-	foreach d $NMEABulkData {
+	foreach d $bulkdata {
 	    set t [CreatePos [lindex $d 3] [lindex $d 4] \
-			DMS latlong "WGS 84"]
+			DMS latlong $NMEADatum]
 	    lappend t [lindex $d 1] [lindex $d 2]
 	    lappend t [lindex $d 5] [lindex $d 6]
 	    lappend tps $t
@@ -326,18 +331,23 @@ proc NMEAPoint {data} {
 
 proc NMEAOff {} {
 
-    global SRLFILE LogFile HaltNMEA NMEABulkData
+    global Datum DataDefault NMEATrackName NMEATrackFamily \
+	NMEABulkData SRLFILE HaltNMEA ListExists \
+	TrackCountChannel TrackCount Type
 
     set HaltNMEA 1
-    if {[llength $NMEABulkData] > 0} {NMEAInDataTR}
     close $SRLFILE
-    CloseNMEAProgrWindow
-    return
-}
-
-proc CloseNMEAProgrWindow {} {
-
-    destroy .2inprogr
+    if {$ListExists} {ClearNMEAList}
+    if {[llength $NMEABulkData] > 0} {NMEAInDataTR $NMEATrackName \
+	$NMEABulkData}
+    if {$Type} {
+	WriteLogFile $NMEATrackName
+    } else {
+	WriteLogFile $NMEATrackFamily
+	puts $TrackCountChannel $TrackCount
+	close $TrackCountChannel
+    }
+    set DataDefault(TR) [list "" "" $Datum "" "" 0]
     return
 }
 
@@ -370,7 +380,6 @@ proc NMEAMapNameWindow {} {
     wm geometry .mapname -10+0
 
     frame .mapname.fr -relief flat -borderwidth 0 -bg $COLOUR(messbg)
-#    tk scaling -displayof .mapname 1
     label .mapname.fr.txt -text $MapName -font {charter 24 bold}
     pack .mapname.fr -side right -pady 0
     pack .mapname.fr.txt -side left -pady 0
@@ -379,32 +388,6 @@ proc NMEAMapNameWindow {} {
     return 0
 }
 
-proc NMEAInProgrWindow {} {
-
-    # create dialog for signaling operation in progress
-    # fail if window already exists
-    #  single button: Abort; no bindings
-
-    global COLOUR EPOSX EPOSY TXT Type
-
-    if { [winfo exists .2inprogr] } { Raise .2inprogr ; bell ; return 1 }
-    toplevel .2inprogr
-    wm protocol .2inprogr WM_DELETE_WINDOW { bell }
-    wm title .2inprogr $TXT(Gman)
-    wm transient .2inprogr
-    wm geometry .2inprogr +0+0
-
-    frame .2inprogr.fr -relief flat -borderwidth 1 -bg $COLOUR(messbg)
-    label .2inprogr.fr.txt -text $TXT(Nmealogpgr)
-    button .2inprogr.fr.stop -text $TXT(stop) -default active -command "NMEAOff"
-    pack .2inprogr.fr -side left -pady 0
-    pack .2inprogr.fr.txt .2inprogr.fr.stop -side left -pady 2
-    update idletasks
-    if {! $Type} {grab .2inprogr.fr.stop}
-    RaiseWindow .2inprogr
-    return 0
-}
-
 proc NMEASetupWindow {} {
 
     global TXT LOGMENU LOGCMD NMEATrackName
@@ -415,50 +398,41 @@ proc NMEASetupWindow {} {
     wm title .nmea "GPS Manager - NMEA"
     wm geometry .nmea +200+200
 
-    frame .nmea.fr0 -relief flat -width 175 -borderwidth 5 
-    frame .nmea.fr1 -relief flat -width 175 -borderwidth 5 
-    frame .nmea.fr2 -relief flat -width 175 -borderwidth 5  
-    frame .nmea.fr3 -relief flat -width 175 -borderwidth 5
-    frame .nmea.fr4 -relief flat -width 175 -borderwidth 5
-    label .nmea.fr0.title -text $TXT(NMEAauto)
-    label .nmea.fr1.title -text $TXT(NMEApreload)
-    label .nmea.fr2.title -text $TXT(Autoload)
-    label .nmea.fr3.title -text $TXT(NMEASet)
-    label .nmea.fr4.title -text $TXT(Newtrack)
-    button .nmea.fr2.auto -text $TXT(load) -relief raised -default normal \
-	-command LoadAutoMapInfo
-    button .nmea.fr1.logr -text $TXT(start) -relief raised -default normal \
-	-command "StartNMEALog 1"
-    button .nmea.fr0.logr -text $TXT(start) -relief raised -default normal \
-	-command "StartNMEALog 0"
-    menubutton .nmea.fr3.interval -relief raised -width 10 \
+    pack [frame .nmea.fr0 -relief flat -width 175 -borderwidth 5] 
+    pack [label .nmea.fr0.title -text $TXT(NMEASet)] -pady 2
+    menubutton .nmea.fr0.interval -relief raised -width 10 \
 	-text $TXT(Interval) -direction left -textvariable LogInterval \
-	-menu .nmea.fr3.interval.m
-    menu .nmea.fr3.interval.m -tearoff 0
-    wm transient .nmea.fr3.interval.m .nmea.fr3.interval
+	-borderwidth 5 -menu .nmea.fr0.interval.m
+    pack .nmea.fr0.interval -pady 2
+    menu .nmea.fr0.interval.m -tearoff 0
+    wm transient .nmea.fr0.interval.m .nmea.fr0.interval
     foreach t $LOGMENU c $LOGCMD {
-	.nmea.fr3.interval.m add command -label $t \
+	.nmea.fr0.interval.m add command -label $t \
 	-command "set NMEALogInterval $c"
     }
-    entry .nmea.fr4.track -relief sunken -bd 2 -state normal -width 10 \
+    pack [frame .nmea.fr1 -relief flat -width 175 -borderwidth 5] 
+    pack [frame .nmea.fr1.fr0 -relief flat -width 175 -borderwidth 5] -side left  
+    pack [frame .nmea.fr1.fr1 -relief flat -width 175 -borderwidth 5] -side right
+    pack [frame .nmea.fr1.fr0.fr0 -relief flat -width 175 -borderwidth 5]
+    pack [frame .nmea.fr1.fr0.fr1 -relief flat -width 175 -borderwidth 5]
+    pack [frame .nmea.fr1.fr1.fr0 -relief flat -width 175 -borderwidth 5]
+    pack [frame .nmea.fr1.fr1.fr1 -relief flat -width 175 -borderwidth 5]
+    pack [label .nmea.fr1.fr0.fr0.title -text $TXT(Autoload)] -pady 2
+    button .nmea.fr1.fr0.fr0.auto -text $TXT(load) -relief raised -default normal \
+	-command LoadAutoMapInfo
+    pack .nmea.fr1.fr0.fr0.auto -pady 2
+    pack [label .nmea.fr1.fr0.fr1.title -text $TXT(NMEAauto)] -pady 2
+    button .nmea.fr1.fr0.fr1.logr -text $TXT(start) -relief raised -default normal \
+	-command "StartNMEALog 0"
+    pack .nmea.fr1.fr0.fr1.logr -pady 2
+    pack [label .nmea.fr1.fr1.fr0.title -text $TXT(Newtrack)] -pady 4
+    entry .nmea.fr1.fr1.fr0.track -relief sunken -bd 5 -state normal -width 10 \
 	-textvariable NMEATrackName
-#    .nmea.fr4.track insert 0 $NMEATrackName
-    bind .nmea.fr4.track <Return> NMEAUpdate    
-    pack .nmea.fr0 -side top -pady 2
-    pack .nmea.fr1 -side top -pady 2
-    pack .nmea.fr2 -side top -pady 2
-    pack .nmea.fr3 -side top -pady 2
-    pack .nmea.fr4 -side top -pady 2
-    pack .nmea.fr0.title -side top -pady 2
-    pack .nmea.fr1.title -side top -pady 2
-    pack .nmea.fr2.title -side top -pady 2
-    pack .nmea.fr3.title -side top -pady 2
-    pack .nmea.fr4.title -side top -pady 2
-    pack .nmea.fr2.auto -side bottom -pady 2
-    pack .nmea.fr1.logr -side bottom -pady 2
-    pack .nmea.fr0.logr -side bottom -pady 2
-    pack .nmea.fr3.interval -side bottom -pady 2
-    pack .nmea.fr4.track -side bottom -pady 2
+    pack .nmea.fr1.fr1.fr0.track -pady 4
+    pack [label .nmea.fr1.fr1.fr1.title -text $TXT(NMEApreload)] -pady 4
+    button .nmea.fr1.fr1.fr1.logr -text $TXT(start) -relief raised -default normal \
+	-command "StartNMEALog 1"
+    pack .nmea.fr1.fr1.fr1.logr -pady 4
     return 0
 }
 
@@ -516,7 +490,9 @@ proc TimeStamp {time} {
 
 proc MapControl {lat long} {
 
-    global LogFile SRLFILE NMEANoImage FirstFlag Top Bottom Right Left
+    global LogFile SRLFILE NMEANoImage FirstFlag Top Bottom Right \
+	Left CurrentPath NMEABulkData NMEATrackName NMEATrackFamily \
+	NMEANoImageTrackFamily TRName
 
     set within 1
     if {$FirstFlag} {
@@ -524,10 +500,15 @@ proc MapControl {lat long} {
 	if {$path == ""} {
 	    if {! $NMEANoImage} {
 		NMEANoImageMess
+		set NMEATrackFamily $NMEANoImageTrackFamily
+		set NMEATrackName [SetUpTrackName $NMEATrackFamily]
 	    }
 	} else {
 	    close $SRLFILE
+	    set NMEATrackFamily [SetUpTrackFamily $path]
+	    set NMEATrackName [SetUpTrackName $NMEATrackFamily]
 	    LoadIndexedMap $path
+	    set NMEAFirstLine 1
 	    OpenNMEAPort
 	    set FirstFlag 0
 	}
@@ -539,23 +520,89 @@ proc MapControl {lat long} {
 	if {! $within} {
 	    set path [FindMapPath $lat $long]
 	    if {$path == ""} {
-		if {! $NMEANoImage} {NMEANoImageMess}
-#		set FirstFlag 1
+		if {! $NMEANoImage} {
+		    NMEALogSetup $NMEABulkData
+		    NMEANoImageMess
+		}
+		set NMEATrackFamily $NMEANoImageTrackFamily
+		set NMEATrackName [SetUpTrackName $NMEATrackFamily]
 	    } else {
 		close $SRLFILE
+		NMEALogSetup $NMEABulkData
+		set lastlog $NMEATrackName
+		set NMEATrackFamily [SetUpTrackFamily $path]
+		set NMEATrackName [SetUpTrackName $NMEATrackFamily]
 		LoadIndexedMap $path
 		set NMEANoImage 0
 		destroy .noimg
 		OpenNMEAPort
+		ToggleDisplayNamed TR $lastlog
 	    }
 	}
     }
     return
 }
 
+proc NMEALogSetup {bulkdata} {
+
+    global NMEABulkData ListExists NMEATrackName \
+	NMEATrackFamily NMEALastLogLength
+
+    set tally [llength $bulkdata]
+    if {[llength $bulkdata] > 0} {
+	if {$ListExists} {ClearNMEAList}			
+	NMEAInDataTR $NMEATrackName $bulkdata
+	set ListExists 1
+    }
+    WriteLogFile $NMEATrackFamily
+    if {$tally > $NMEALastLogLength} {
+        set first [expr $tally - $NMEALastLogLength]
+        set bulkdata [lreplace $bulkdata 0 $first]
+	ClearNMEAList
+	NMEAInDataTR $NMEATrackName $bulkdata
+	set ListExists 1
+    }
+    set NMEABulkData ""
+}
+
+proc SetUpTrackName {family} {
+
+    global NMEANoImage TrackCount NMEANoImageTrackFamily
+
+    append suffix $family "-" $TrackCount
+    if {$family != $NMEANoImageTrackFamily} \
+	{set TrackCount [incr TrackCount]}
+    if {$TrackCount > 999} {set TrackCount 0}
+    return $suffix
+}
+ 
+proc SetUpTrackFamily {path} {
+
+    set slash [string last / $path]
+    set slash [expr $slash + 1]
+    set indx [string length $path]
+    set indx [expr $indx-5]
+    return [string range $path $slash $indx] 
+
+}
+
+proc WriteLogFile {track} {
+
+    set ix [IndexNamed TR $track]
+    append fname $track ".trk"
+    set exists [file exists $fname]
+    if {$exists} {
+	set id [open $fname a ]
+    } else {
+	set id [open $fname w ]
+    }
+    SaveTrackFile $id $fname
+    return    
+} 
+
 proc FindMapPath {lat long} {
 
-    global ImageIndex Top Bottom Right Left MapName
+    global ImageIndex Top Bottom Right Left MapName 
 
     destroy .mapname
     set max [array size ImageIndex]
@@ -581,3 +628,108 @@ proc FindMapPath {lat long} {
     }
     return
 }
+
+proc ClearNMEAList {} {
+    # forget all items in NMEA log
+    global ListInds
+
+    foreach i $ListInds(TR) { Forget TR $i }
+    if { "$ListInds(TR)" == "" } {ChangeOnStateList TR disabled}
+    return
+}
+
+proc SetTrackCountChannel {} {
+
+    # Variable "TrackCount" used to avoid duplicate track names.
+
+    global USERDIR TrackCountFile TrackCount TrackCountChannel
+
+    append fname $USERDIR "/" $TrackCountFile
+    if { [file exists $fname] } {
+	set ch [open $fname r]
+	gets $ch TrackCount
+	if {$TrackCount == ""} {set TrackCount 0}
+	close $ch
+    } else {set TrackCount 0}
+    set TrackCountChannel [open $fname w]
+}
+
+proc SaveTrackFile {f fname} {
+    # Hacked from proc SaveFileTo.
+    # save information to file $f in GPSMan format
+    # if $f=="" ask user to select output file
+    # see proc SaveFile for description of the arguments
+    global SFilePFrmt SFilePFType SFileDatum SFileHeader Storage TXT TYPES \
+	    Number
+
+    set ids [lindex $Storage(TR) 0]
+    global $ids
+    set ixs [array names $ids]
+    set lp [list [list TR $ixs]]
+    set SFileHeader($f) 0
+    foreach p $lp {
+	SaveTR1 $f [lindex $p 1] $fname
+    }
+    close $f
+    return
+}
+
+proc SaveTR1 {file ixs fname} {
+    # save data for TRs with indices in list $ixs to file in GPSMan format
+    global FCOMMAND TRName TRObs TRDatum TRTPoints TRHidden DataIndex \
+	    SFileDatum
+
+    WriteHeader1 $file DMS $fname
+    set ilt $DataIndex(TPlatDMS) ; set ilg $DataIndex(TPlongDMS)
+    set idt $DataIndex(TPdate)
+    set ial $DataIndex(TPalt) ; set idp $DataIndex(TPdepth)
+    foreach i $ixs {
+	if { "$TRDatum($i)" != "$SFileDatum($file)" } {
+	    WriteChgDatum $file $TRDatum($i)
+	}
+	puts -nonewline $file [format "%s\t%s" $FCOMMAND(TR) $TRName($i)]
+	foreach h $TRHidden($i) {
+	    puts -nonewline $file "\t$h"
+	}
+	puts $file ""
+	SaveNB $file $TRObs($i)
+	foreach tp $TRTPoints($i) {
+	    set alt [lindex $tp $ial] ; set dep [lindex $tp $idp]
+	    if { "$dep" == "" } {
+		if { "$alt" == "" } {
+		    puts $file [format "\t%s\t%s\t%s" [lindex $tp $idt] \
+			    [lindex $tp $ilt] [lindex $tp $ilg]]
+		} else {
+		    puts $file [format "\t%s\t%s\t%s\t%s" [lindex $tp $idt] \
+			    [lindex $tp $ilt] [lindex $tp $ilg] $alt]
+		}
+	    } else { 
+		puts $file [format "\t%s\t%s\t%s\t%s\t%s" [lindex $tp $idt] \
+			[lindex $tp $ilt] [lindex $tp $ilg] $alt $dep]
+	    }
+	}
+	puts $file ""
+    }
+    return
+}
+
+proc WriteHeader1 {file pformt fname} {
+    # write header to file in GPSMan format, using $pformt for positions
+    global TimeOffset Datum FCOMMAND CREATIONDATE \
+	    SFilePFrmt SFilePFType SFileDatum SFileHeader MESS
+
+    if {[file size $fname] ==0 } {
+	puts $file "$FCOMMAND(comment) $MESS(written) GPSManager [NowTZ]"
+	puts $file "$FCOMMAND(comment) $MESS(editrisk)"
+	puts $file ""
+	puts $file "$FCOMMAND(format) $pformt $TimeOffset $Datum"
+	puts $file "$FCOMMAND(dates) $FCOMMAND($CREATIONDATE)"
+	puts $file ""
+    }
+	set SFilePFrmt($file) $pformt
+	set SFilePFType($file) [PosType $pformt]
+	set SFileDatum($file) $Datum
+	set SFileHeader($file) 1
+    }
+    return
+}
