@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: main.tcl
-#  Last change:  27 July 2006
+#  Last change:  14 June 2007
 #
 # Includes contributions by
 #  - Brian Baulch (baulchb@onthenet.com.au) marked "BSB contribution"
@@ -26,9 +26,9 @@
 #  - David Gardner (djgardner@users.sourceforge.net) marked "DJG contribution"
 #
 
-set VERSIONDATE(day) 27
-set VERSIONDATE(month) 7
-set VERSIONDATE(year) 2006
+set VERSIONDATE(day) 14
+set VERSIONDATE(month) 6
+set VERSIONDATE(year) 2007
 
       # these tests here prevent errors in the lang*.tcl files
 if { [lsearch -exact "KM NAUTMILE STATMILE" $DISTUNIT] == -1 } {
@@ -137,7 +137,7 @@ if { int($TimeOffset) != $TimeOffset } {
     exit 1
 }
 
-set VERSION 6.3.1
+set VERSION 6.3.2
 
 set WindowStack ""
 
@@ -220,7 +220,7 @@ InitImages
 ### on Unix systems GPSMan cannot be executed as root for security reasons
 ###  this can be disabled by replacing the condition in the if by  0  (zero)
 if { $UNIX && $env(USER) == "root" } {
-    GMMessage $MESS(cantexecasroot)
+    GMMessage $MESS(cantexecasroot) wait
     exit 1
 }
 
@@ -228,7 +228,7 @@ if { $UNIX && $env(USER) == "root" } {
 if { $InitDir } {
     if { ! [GMConfirm [format $MESS(oktomkdir) $USERDIR]] || \
 	    [catch {file mkdir $USERDIR}] } {
-	GMMessage [format $MESS(createdir) $USERDIR]
+	GMMessage [format $MESS(createdir) $USERDIR] wait
 	exit 1
     }
 }
@@ -236,7 +236,7 @@ if { $InitDir } {
 ### if no preferences file force one being written
 if { $InitOpts || [OutDatedOptions] } {
     if { [SetOptions] } {
-	GMMessage $MESS(goingdown)
+	GMMessage $MESS(goingdown) wait
 	exit 0
     }
     exit 1
