@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2005 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: maptransf.tcl
-#  Last change:  17 May 2005
+#  Last change:  16 April 2007
 #
 
 ## Uses information kindly supplied by
@@ -42,7 +42,7 @@
 ##
 
 ## when adding new transformation procedures the variables MAPTRANSFNPTS and
-#   MAPKNOWNTRANSFS must be changed in gpsman.tcl
+#   MAPKNOWNTRANSFS must be changed in metadata.tcl
 # see also relevant procedures in map.tcl, command_parse.tcl, command.tcl
 
      # indices of MTData array used for each transformation
@@ -348,6 +348,100 @@ proc GaussReduce {n} {
     return 1
 }
 
+##### least-squares fit
+# from the equations provided by James B. Mehl
+#  with origin of pixel coordinates in the upper left corner of image
+#
+# 	(xt,yt) terrain coordinates
+# 	(xm,-ym) map coordinates (pixel, ym grows downwards!)
+#
+# 	(e,f) terrain coordinates of upper left corner, map (0,0)
+#
+# 	(xt,yt) = lambda (xm,ym) + (e,f)
+#
+# 	(e,f) = lambda (xm_0,H-ym_0) + (xt_0,yt_0)
+#         where H is map height and (xt_0,yt_0) the terrain coordinates of
+#         map lower corner
+#
+#       for all control points replace vertical coordinate ym_i by
+#         H-ym_i
+#
+# 	f = sum_i (xt_i-xt_0-lambda xm_i)^2+(yt_i-yt_0-lambda ym_i)^2
+# 	min
+#
+# 	d(f,xt_0)=0 =>   S_xt - N xt_0 - lambda S_xm = 0
+# 		=>   xt_0 = (S_xt - lambda S_xm) / N
+# 	d(f,yt_0)=0 =>   S_yt - N yt_0 - lambda S_ym = 0
+# 		=>   yt_0 = (S_yt - lambda S_ym) / N
+# 	d(f,lambda)=0 =>   S_xtm - xt_0 S_xm - lambda S_xmm +
+# 				S_ytm - y_t0 S_ym - lambda S_ymm
+#       => lambda = (N (S_xtm + S_ytm) - S_xt S_xm - S_yt S_ym) /
+# 			(N (S_xmm + S_ymm) - S_xm^2 - S_ym^2)
+#
+
+proc MapInitLeastSquaresTransf {args} {
+    # use datum and geodetic and pixel coordinates of points either
+    #  from the graphical interface or from a GPSMan least-squares
+    #  info file to compute parameters of affine conformal
+    #  transformation with no rotation using a least-squares fit
+    #  $args==""  points are WPs, with $MapLoadWPs the list of
+    #          indices of relevant WPs, used by proc MapGeoRefPoints
+    #          that sets MapLoadPos(_,_) to the pixel coordinates
+    #       ==pair whose 2nd element is the projection and 1st elemet is
+    #          a pair with list of latd,longd,datum, and list of pairs with
+    #          pixel coordinates (x, y, in Tcl sign convention with origin
+    #          at upper left corner)
+    # return 0 on error or operation being cancelled
+    global MPData MTData MapScale MPData MapLoadWPs MapLoadPos MapImageHeight
+
+    if { $args == "" } {
+	set n [llength $MapLoadWPs]
+	if { [set tcs [MapGeoRefPoints $n]] == -1 } { return 0 }
+	set xys ""
+	for { set k 0 } { $k < $n } { incr k } {
+	    lappend xys [list $MapLoadPos($k,x) $MapLoadPos($k,y)]
+	}
+    } else {
+	foreach "p proj" $args { break }
+	foreach "llds xys" $p { break }
+	set tcs ""
+	foreach tr $llds {
+	    foreach "latd longd datum" $tr {}
+	    lappend tcs [Proj${proj}Point MPData $latd $longd $datum]
+	}
+	set n [llength $llds]
+    }
+    # least-squares on $tcs and $xys
+    foreach i "xt yt xm ym xtm ytm xtt ytt xmm ymm" { set s($i) 0 }
+    foreach tc $tcs pc $xys {
+	foreach "xt yt" $tc {}
+	foreach "xm ym" $pc {}
+	set ym [expr $MapImageHeight-$ym]
+	foreach i "xt yt xm ym" {
+	    set s($i) [expr $s($i)+[set $i]]
+	}
+	foreach i "x y" {
+	    set s(${i}tm) [expr $s(${i}tm)+[set ${i}t]*[set ${i}m]]
+	    set s(${i}tt) [expr $s(${i}tt)+[set ${i}t]*[set ${i}t]]
+	    set s(${i}mm) [expr $s(${i}mm)+[set ${i}m]*[set ${i}m]]
+	}
+    }
+    MapTransfIs NoRot
+    if { [catch {set k [expr 1.0*($n*($s(xtm)+$s(ytm)) - $s(xt)*$s(xm) - \
+				      $s(yt)*$s(ym)) /  \
+			    ($n*($s(xmm)+$s(ymm)) - $s(xm)*$s(xm) - \
+				 $s(ym)*$s(ym))]}] } {
+	return 0
+    }
+    # assuming:
+    # set MapLoadPos(origin,x) 0
+    # set MapLoadPos(origin,y) 0
+    set MTData(lambda) [set MapScale $k]
+    set MTData(e) [expr 1.0*($s(xt)-$k*$s(xm))/$n]
+    set MTData(f) [expr 1.0*($s(yt)-$k*$s(ym))/$n+$k*$MapImageHeight]
+    return 1
+}
+
 ##### TFW metadata file
 # there is no publicly available definition for this format:
 #   each file has 6 lines with a number each referred to by tfw_ below
