@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: metadata.tcl
-#  Last change:  25 July 2006
+#  Last change:  4 May 2007
 #
 
 # abstract data used in command-mode, in configuring the graphical interface,
@@ -55,7 +55,7 @@
 array set FILEFORMAT {
     GPSMan,mode           {in out}
     GPSMan,filetype       data
-    GPSMan,types          {WP RT TR LN LAP}
+    GPSMan,types          {WP RT TR LN LAP GR}
     GPStrans,mode         {in out}
     GPStrans,filetype     single
     GPStrans,types        {WP RT TR}
@@ -187,8 +187,22 @@ array set FILEFORMAT {
     Ozi,filetype          single
     Ozi,types             {WP TR}
     Ozi,GREls             1
+    SimpleText,mode       {in out}
+    SimpleText,filetype   unique
+    SimpleText,types      TR
 }
 
+# compute names of defined formats
+
+set ffs ""
+
+foreach n [array names FILEFORMAT *,mode] {
+    regsub {,mode$} $n "" n
+    lappend ffs $n
+}
+
+set FILEFORMAT(names) $ffs
+
 ### receiver protocols with external and internal representations
 
 array set RECPROTS {
@@ -207,7 +221,11 @@ array set MAPTRANSFNPTS {
     Affine     3
     AffineConf 2
     NoRot      3
+    LeastSquares   >=3
 }
 
+# LeastSquares not added to MAPKNOWNTRANSFS as it is not practical to
+#  give a large number of points in the command line
+
 set MAPKNOWNTRANSFS {Affine AffineConf NoRot}
 
