@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2005 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: navigate.tcl
-#  Last change:  26 May 2005
+#  Last change:  27 July 2006
 #
 
 ##### travel/navigation display
@@ -1305,8 +1305,8 @@ proc TravelData {hms posn lat long fix alt velx vely vel_z trk speed} {
 	if { $speed != "_" && $trk != "_" } {
 	    set a [expr $trk*0.01745329251994329576]
 	    # m/s
-	    set velx [expr $speed*cos($a)/3.6]
-	    set vely [expr $speed*sin($a)/3.6]
+	    set velx [expr $speed*sin($a)/3.6]
+	    set vely [expr $speed*cos($a)/3.6]
 	} elseif { $dtime < $Travel(nav,maxdtime) } {
 	    # assume $Travel(prevtime) != -1 because $RealTimeLogLast
 	    #  is much larger than $Travel(nav,maxdtime)
