@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2005 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: options.tcl
-#  Last change:  12 September 2004
+#  Last change:  5 July 2005
 #
 
 # when inserting new data here, define TXT(opt*)
@@ -71,13 +71,13 @@ set OPTMAXHEIGHT 10
 
 set OPTSTRUCT {   GPSREC _GPSRecConf _Formats _sep
                   _Data _sep
-	          _Interf _Geom COLOUR _MapGeom MAPCOLOUR _sep 
+	          _Interf _Geom COLOUR _MapGeom MAPCOLOUR _Fonts _sep 
                   _Files
 }
 
 set OPTTYPE(_Interf) group
 set OPTGRPL(_Interf) {MWINDOWSCONF LANG ISOLATIN1 DELETE USESLOWOPWINDOW
-                      BalloonHelp TRNUMBERINTVL LNSREACT}
+                      BalloonHelp TRNUMBERINTVL TRINFO LNSREACT}
 set OPTTYPE(MWINDOWSCONF) fixedtextmenu
 set OPTMENUCONTS(MWINDOWSCONF) {map lists}
 set OPTMENUNAMES(MWINDOWSCONF) TXT
@@ -91,6 +91,10 @@ set OPTTYPE(DELETE) boolean
 set OPTTYPE(USESLOWOPWINDOW) boolean
 set OPTTYPE(BalloonHelp) boolean
 set OPTTYPE(TRNUMBERINTVL) nat
+set OPTTYPE(TRINFO) fixedtextmenu
+set OPTMENUCONTS(TRINFO) {number date}
+set OPTMENUNAMES(TRINFO) TXT
+set OPTMENUPREFIX(TRINFO) ""
 set OPTTYPE(LNSREACT) boolean
 
 set OPTTYPE(GPSREC) menu
@@ -149,22 +153,43 @@ set OPTTYPE(ACCFORMULAE) boolean
 set OPTTYPE(ASKPROJPARAMS) boolean
 
 set OPTTYPE(_MapGeom) group
-set OPTGRPL(_MapGeom) {MAPWIDTH MAPHEIGHT _sep
+set OPTGRPL(_MapGeom) {MAPWIDTH MAPHEIGHT ICONSIZE _sep
                        MAPSCLENGTH MAPSCALE _sep
-                       DPOSRTMAP _sep
-                       MAPFONTSIZE TRAVELFONTSIZE DEFTRTWIDTH DEFTTRWIDTH
+                       DPOSRTMAP _sep DEFTRTWIDTH DEFTTRWIDTH
                        DEFTLNWIDTH}
+set OPTTYPE(MAPWIDTH) nat
+set OPTTYPE(MAPHEIGHT) nat
+set OPTTYPE(ICONSIZE) fixedmenu
+set OPTMENUCONTS(ICONSIZE) {15x15 30x30}
+set OPTTYPE(MAPSCLENGTH) nat
+set OPTTYPE(MAPSCALE) fixedmenu
+set OPTMENUCONTS(MAPSCALE) $MAPDISTS
+set OPTTYPE(DPOSRTMAP) float
+set OPTTYPE(DEFTRTWIDTH) fixedmenu
+set OPTMENUCONTS(DEFTRTWIDTH) {1 2 3 4 5 6 7 8}
+set OPTTYPE(DEFTTRWIDTH) fixedmenu
+set OPTMENUCONTS(DEFTTRWIDTH) {1 2 3 4 5 6 7 8}
+set OPTTYPE(DEFTLNWIDTH) fixedmenu
+set OPTMENUCONTS(DEFTLNWIDTH) {1 2 3 4 5 6 7 8}
+
+set OPTTYPE(_Fonts) group
+set OPTGRPL(_Fonts) {FONTSIZE FIXEDFONTSIZE MAPFONTSIZE TRAVELFONTSIZE}
+set OPTTYPE(FONTSIZE) fixedmenu
+set OPTMENUCONTS(FONTSIZE) $MAPFTSIZES
+set OPTTYPE(FIXEDFONTSIZE) fixedmenu
+set OPTMENUCONTS(FIXEDFONTSIZE) $MAPFTSIZES
+set OPTTYPE(MAPFONTSIZE) fixedmenu
+set OPTMENUCONTS(MAPFONTSIZE) $MAPFTSIZES
+set OPTTYPE(TRAVELFONTSIZE) fixedmenu
+set OPTMENUCONTS(TRAVELFONTSIZE) $MAPFTSIZES
 
 set OPTTYPE(_Geom) group
 set OPTGRPL(_Geom) {MAXMENUITEMS LISTWIDTH LISTHEIGHT _sep
                     LPOSX LPOSY MPOSX MPOSY RPOSX RPOSY _sep
                     EPOSX EPOSY DPOSX DPOSY}
-set OPTTYPE(MAPWIDTH) nat
-set OPTTYPE(MAPHEIGHT) nat
-set OPTTYPE(MAPSCLENGTH) nat
-set OPTTYPE(MAPSCALE) fixedmenu
-set OPTMENUCONTS(MAPSCALE) $MAPDISTS
 set OPTTYPE(MAXMENUITEMS) nat
+set OPTTYPE(LISTWIDTH) nat
+set OPTTYPE(LISTHEIGHT) nat
 set OPTTYPE(LPOSX) nat
 set OPTTYPE(LPOSY) nat
 set OPTTYPE(MPOSX) nat
@@ -175,19 +200,6 @@ set OPTTYPE(EPOSX) nat
 set OPTTYPE(EPOSY) nat
 set OPTTYPE(DPOSX) nat
 set OPTTYPE(DPOSY) nat
-set OPTTYPE(DPOSRTMAP) float
-set OPTTYPE(MAPFONTSIZE) fixedmenu
-set OPTMENUCONTS(MAPFONTSIZE) $MAPFTSIZES
-set OPTTYPE(TRAVELFONTSIZE) fixedmenu
-set OPTMENUCONTS(TRAVELFONTSIZE) $MAPFTSIZES
-set OPTTYPE(DEFTRTWIDTH) fixedmenu
-set OPTMENUCONTS(DEFTRTWIDTH) {1 2 3 4 5 6 7 8}
-set OPTTYPE(DEFTTRWIDTH) fixedmenu
-set OPTMENUCONTS(DEFTTRWIDTH) {1 2 3 4 5 6 7 8}
-set OPTTYPE(DEFTLNWIDTH) fixedmenu
-set OPTMENUCONTS(DEFTLNWIDTH) {1 2 3 4 5 6 7 8}
-set OPTTYPE(LISTWIDTH) nat
-set OPTTYPE(LISTHEIGHT) nat
 
 set OPTTYPE(COLOUR) array
 set OPTELTYPE(COLOUR) colour
