@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 200 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2007 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,15 +17,15 @@
 #      along with this program.
 #
 #  File: search.tcl
-#  Last change:  21 October 2004
+#  Last change:  29 April 2007
 #
 
 proc Search {} {
     # create dialog for searching items
     global PositionFormat SDatum Datum SearchWidgets SearchDistance \
-	SearchWPName SearchWPSymbol SearchRTWP SearchDate SearchSpec \
-	SearchDomain DEFAULTSYMBOL DPOSX DPOSY COLOUR TXT MESS TYPES \
-	ICONWIDTH ICONHEIGHT SYMBOLIMAGE NAMEWIDTH YEAR0 MONTHNAMES \
+	SearchWPName SearchWPSymbol SearchRTWP SearchDate \
+	SearchSpec SearchDomain DEFAULTSYMBOL DPOSX DPOSY COLOUR TXT MESS \
+	TYPES ICONWIDTH ICONHEIGHT SYMBOLIMAGE NAMEWIDTH YEAR0 MONTHNAMES \
 	MAXMENUITEMS SUPPORTLAPS
 
     set stypes $TYPES
@@ -99,16 +99,15 @@ proc Search {} {
 	    -variable SearchDistance(how) -onvalue fromWP -offvalue fromPos \
 	    -selectcolor $COLOUR(check) -anchor w
     set SearchWPName ""
-    menubutton $w.fr.fWPTR.dist.wpp.wp.it -textvariable SearchWPName \
-	    -relief raised -direction below -width $NAMEWIDTH \
-	    -menu $w.fr.fWPTR.dist.wpp.wp.it.m
-    FillItemsMenu $w.fr.fWPTR.dist.wpp.wp.it.m it WP SearchWP
+    button $w.fr.fWPTR.dist.wpp.wp.it -textvariable SearchWPName \
+	    -width $NAMEWIDTH \
+	    -command "ChItemsCall WP single AssignGlobal SearchWPName"
     frame $w.fr.fWPTR.dist.wpp.p -relief flat -borderwidth 0
     checkbutton $w.fr.fWPTR.dist.wpp.p.b -text $TXT(fromPos): -width 13 \
 	    -variable SearchDistance(how) -onvalue fromPos -offvalue fromWP \
 	    -selectcolor $COLOUR(check) -anchor w
     frame $w.fr.fWPTR.dist.wpp.p.p -relief sunken -borderwidth 1
-    ShowPosnDatum $w.fr.fWPTR.dist.wpp.p.p $PositionFormat "" \
+    ShowPosnDatum $w.fr.fWPTR.dist.wpp.p.p $PositionFormat [list ""] \
 	    SearchChangeDatum SDatum SDatum normal 1 nil
     frame $w.fr.fWPTR.dir -relief flat -borderwidth 0
     label $w.fr.fWPTR.dir.tit -text $TXT(azimuth):
@@ -161,20 +160,20 @@ proc Search {} {
 	    -width 5 -exportselection 1
     bind $w.fr.fWPTR.its.spc.bt.d2 <Leave> { SearchBearingAngle }
 
-    frame $w.fr.fWP -relief flat -borderwidth 0
+    frame $w.fr.fWPsy -relief flat -borderwidth 0
     set SearchSpec(symbol) 0
-    checkbutton $w.fr.fWP.b -text "$TXT(nameWP) $TXT(with)" -anchor w \
+    checkbutton $w.fr.fWPsy.b -text "$TXT(nameWP) $TXT(with)" -anchor w \
 	    -variable SearchSpec(symbol) -onvalue 1 -offvalue 0 \
 	    -selectcolor $COLOUR(check) -state disabled
-    menubutton $w.fr.fWP.symb -text $TXT(symbol) -relief raised \
-	    -direction below -menu $w.fr.fWP.symb.m
-    set mw $w.fr.fWP.symb.m
+    menubutton $w.fr.fWPsy.symb -text $TXT(symbol) -relief raised \
+	    -direction below -menu $w.fr.fWPsy.symb.m
+    set mw $w.fr.fWPsy.symb.m
     menu $mw -tearoff 0
     FillSymbolsMenu $mw SearchChangeSymbol
-    canvas $w.fr.fWP.symbim -width $ICONWIDTH -height $ICONHEIGHT
-    $w.fr.fWP.symbim create image 1 1 -anchor nw \
+    canvas $w.fr.fWPsy.symbim -width $ICONWIDTH -height [expr $ICONHEIGHT+2]
+    $w.fr.fWPsy.symbim create image 1 5 -anchor nw \
 	    -image $SYMBOLIMAGE($SearchWPSymbol)
-    label $w.fr.fWP.symbname -text $TXT(SY$SearchWPSymbol)
+    label $w.fr.fWPsy.symbname -text $TXT(SY$SearchWPSymbol)
 
     frame $w.fr.fRT -relief flat -borderwidth 0
     set SearchSpec(RTWP) 0
@@ -182,10 +181,8 @@ proc Search {} {
 	    -variable SearchSpec(RTWP) -onvalue 1 -offvalue 0 \
 	    -selectcolor $COLOUR(check) -state disabled
     set SearchRTWP ""
-    menubutton $w.fr.fRT.wp -textvariable SearchRTWP \
-	    -relief raised -direction below -width $NAMEWIDTH \
-	    -menu $w.fr.fRT.wp.m
-    FillItemsMenu $w.fr.fRT.wp.m wp WP SearchRTWP
+    button $w.fr.fRT.wp -textvariable SearchRTWP -width $NAMEWIDTH \
+	    -command "ChItemsCall WP single AssignGlobal SearchRTWP"
 
     if { $SUPPORTLAPS } {
 	set trlaps "TR LAP"
@@ -200,15 +197,14 @@ proc Search {} {
     foreach wh $trlaps {
 	set f $w.fr.f$wh
 	frame $f -relief flat -borderwidth 0
-	set SearchSpec(date) 0
+	set SearchSpec($wh,date) 0
 	checkbutton $f.b -text "$TXT(name$wh) $TXT(started)" -anchor w \
 	    -variable SearchSpec($wh,date) -onvalue 1 -offvalue 0 \
 	    -selectcolor $COLOUR(check) -state disabled
 	foreach de "d m y" vs [list $ds $MONTHNAMES $ys] {
 	    set SearchDate($wh,$de) ""
 	    menubutton $f.$de -textvariable SearchDate($wh,$de) \
-		-relief raised -direction below -width $NAMEWIDTH \
-		-menu $f.$de.m
+		-relief raised -direction below -width 5 -menu $f.$de.m
 	    menu $f.$de.m -tearoff 0
 	    set mw $f.$de.m
 	    $f.$de.m add command -label "($TXT(any))" \
@@ -226,7 +222,7 @@ proc Search {} {
 	}
 	set SearchWidgets($wh) $w.fr.f$wh.b
     }
-    set SearchWidgets(WP) "$w.fr.fWP.b TR/$w.fr.fWPTR.its.tit"
+    set SearchWidgets(WP) "$w.fr.fWPsy.b TR/$w.fr.fWPTR.its.tit"
     set SearchWidgets(RT) "$w.fr.fRT.b"
     lappend SearchWidgets(TR) WP/$w.fr.fWPTR.its.tit
     set SearchWidgets(LN) ""
@@ -258,13 +254,14 @@ proc Search {} {
     pack $w.fr.fWPTR.its.tit $w.fr.fWPTR.its.spc -side left -padx 2
     pack $w.fr.fWPTR.dist $w.fr.fWPTR.dir $w.fr.fWPTR.its -side top -anchor w
 
-    pack $w.fr.fWP.b $w.fr.fWP.symb $w.fr.fWP.symbim $w.fr.fWP.symbname \
-	    -side left -padx 2
+    pack $w.fr.fWPsy.b $w.fr.fWPsy.symb $w.fr.fWPsy.symbim \
+	$w.fr.fWPsy.symbname -side left -padx 2
 
     pack $w.fr.fRT.b $w.fr.fRT.wp -side left -padx 2
 
     pack $w.fr.tit $w.fr.wh $w.fr.dom $w.fr.strs -side top -pady 5
-    pack $w.fr.fWPTR $w.fr.fWP $w.fr.fRT -side top -pady 5 -anchor w
+    pack $w.fr.fWPTR $w.fr.fWPsy $w.fr.fRT -side top -pady 5 \
+	-anchor w
     foreach wh $trlaps {
 	set f $w.fr.f$wh
 	pack $f.b $f.d $f.m $f.y -side left -padx 2
@@ -363,8 +360,8 @@ proc SearchDescription {} {
     #  by global variables (see proc SearchForAny) and the state of the
     #  search window
     global SearchDomain SearchDomGR GRName SearchSpec SearchPattern \
-	    SearchDistance SearchWPName SDatum SearchWPSymbol SearchRTWP \
-	    SearchDate TXT
+	    SearchDistance SearchWPName SDatum SearchWPSymbol \
+	    SearchRTWP SearchDate TXT SUPPORTLAPS
 
     if { $SearchDomain == "GR" } {
 	set ddomgr "" ; set p ""
@@ -426,14 +423,14 @@ proc SearchDescription {} {
 	set f ";\n"
     }
     set dlapdt ""
-    if { $SearchSpec(LAP,date) } {
+    if { $SUPPORTLAPS && $SearchSpec(LAP,date) } {
 	set dlapdt \
       "${f}$TXT(date) $SearchDate(LAP,d) $SearchDate(LAP,m) $SearchDate(LAP,y)"
 	set f ";\n"
     }
     if { $f != "" } { set f "\n" }
     return \
-	"${ddom}\n${dstrs}${fstrs}${ddist}${dsymb}${drtwp}${dtrdt}${dlapdt}.$f"
+ "${ddom}\n${dstrs}${fstrs}${ddist}${dsymb}${drtwp}${dtrdt}${dlapdt}.$f"
 }
 
 proc SearchFor {whs} {
@@ -491,8 +488,9 @@ proc SearchForAny {ids ixs wh} {
     #     $SearchDate($wh,m) month name or ""
     #     $SearchDate($wh,y) year or ""
     # return list of names
-    global SearchSpec SearchPattern SDatum SearchWPSymbol SearchRTWP \
-	    SearchDate WPPosn WPDatum RTWPoints TRTPoints TRDatum LAPStart
+    global SearchSpec SearchPattern SDatum SearchWPSymbol \
+	    SearchRTWP SearchDate WPPosn WPDatum RTWPoints TRTPoints TRDatum \
+	    LAPStart
 
     if { $ixs == "" } { return "" }
     foreach s "name rmrk" a "$ids ${wh}Obs" {
@@ -507,9 +505,7 @@ proc SearchForAny {ids ixs wh} {
 		set ixs [SearchFilter WPCommt $ixs $SearchPattern(cmmt)]
 	    }
 	    if { $SearchSpec(symbol) } {
-		# this is safe only if symbol names have no meta-chars
-		#  used in "string match"
-		set ixs [SearchFilter WPSymbol $ixs $SearchWPSymbol]
+		set ixs [SearchFilterId WPSymbol $ixs $SearchWPSymbol]
 	    }
 	    if { $SearchSpec(dist) } {
 		SetDatumData $SDatum
@@ -631,6 +627,19 @@ proc SearchFilter {arr ixs patt} {
     return $nixs
 }
 
+proc SearchFilterId {arr ixs str} {
+    # select indices from $ixs in array $arr whose contents is $str
+    global $arr
+
+    set nixs ""
+    foreach ix $ixs {
+	if { [set [set arr]($ix)] == $str } {
+	    lappend nixs $ix
+	}
+    }
+    return $nixs
+}
+
 proc SearchResults {lp} {
     # present search results described by list of pairs: type, names
     # this is currently done by creating and opening a group
@@ -723,15 +732,6 @@ proc SearchChangeDatum {datum args} {
     return
 }
 
-proc GMSearchWPChange {menubutton name} {
-    # change WP name in WP/TR search window
-    # name of proc reflects use of FillItemsMenu
-    global SearchWPName
-
-    set SearchWPName $name
-    return
-}
-
 proc SearchBearingAngle {} {
     # change suggested default for bearing opening-angle in WP/TR search window
     global SearchDistance
@@ -758,25 +758,17 @@ proc SearchBearingAngle {} {
     return
 }
 
-proc GMSearchRTWPChange {menubutton name} {
-    # change WP name in RT search window
-    # name of proc reflects use of FillItemsMenu
-    global SearchRTWP
-
-    set SearchRTWP $name
-    return
-}
-
 proc SearchChangeSymbol {symbol args} {
     # change symbol of search window
     #  $args not used, but called back like this
     global SearchWPSymbol SYMBOLIMAGE TXT
 
     set SearchWPSymbol $symbol
-    set fr .gmsearch.fr.fWP
+    set fr .gmsearch.fr.fWPsy
     $fr.symbim delete all
-    $fr.symbim create image 0 0 -anchor nw -image $SYMBOLIMAGE($symbol)
+    $fr.symbim create image 1 5 -anchor nw -image $SYMBOLIMAGE($symbol)
     $fr.symbname configure -text $TXT(SY$symbol)
     return
 }
 
+
