@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: serial.tcl
-#  Last change:  15 March 2006
+#  Last change:  3 June 2006
 #
 # Includes contributions by Matt Martin (matt.martin@ieee.org)
 #  marked "MGM contribution"
@@ -133,6 +133,7 @@ proc OpenSerialLog {} {
 	set d [FormatDay $DateFormat $VERSIONDATE(year) $VERSIONDATE(month) \
 		         $VERSIONDATE(day)]
 	Log "GPSMan $VERSION / $d"
+	Log "Date: [NowTZ]"
 	Log "tcl_version = $tcl_version"
 	foreach f "platform os osVersion machine byteOrder" {
 	    if { ! [catch {set v $tcl_platform($f)}] } {
@@ -323,8 +324,8 @@ proc EndConnCheck {messid} {
     global MYGPS Request MESS CurrPSPID NotLogging GPSOpResult
 
     CloseInProgrWindow
-    if { $MYGPS == "Garmin" } {
-	SymbolsDOForProtocol $CurrPSPID(WPData)
+    if { $MYGPS == "Garmin" && ! [catch {set pid $CurrPSPID(WPData)}] } {
+	SymbolsDOForProtocol $pid
     }
     ResetSerial
     Log "ECC> $MESS($messid)"
@@ -387,18 +388,25 @@ proc DoGetGPS {wh} {
     if { [FailsInProgrWindow $MESS(get$wh)] } { return }
     InitWPRenaming
     set Request get$wh
-    SendData get $wh
+    SendData get $wh 
     return
 }
 
 proc PutGPS {wh ixs args} {
     #  $args used for passing other information, for instance, WP categories
-    global Request MESS PkLastPID
+    # if global GPSOpResult is set to 1 (error) by proc SendData close
+    #  progress window
+    global Request MESS PkLastPID GPSOpResult
 
     set PkLastPID -1
     if { [FailsInProgrWindow $MESS(put$wh)] } { return }
     set Request put$wh
+    set GPSOpResult 0
     SendData put $wh $ixs $args
+    if { $GPSOpResult } {
+	CloseInProgrWindow
+	ResetSerial
+    }
     return
 }
 
