@@ -1,7 +1,7 @@
 #
 #  gpsman --- GPS Manager: a manager for GPS receiver data
 #
-#  Copyright (c) 2004 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
+#  Copyright (c) 2006 Miguel Filgueiras (mig@ncc.up.pt) / Universidade do Porto
 #
 #    This program is free software; you can redistribute it and/or modify
 #      it under the terms of the GNU General Public License as published by
@@ -17,7 +17,7 @@
 #      along with this program.
 #
 #  File: util.tcl
-#  Last change:  7 October 2004
+#  Last change:  18 February 2006
 #
 # Includes contributions by Brian Baulch (baulchb@onthenet.com.au)
 #  marked "BSB contribution"
@@ -25,11 +25,12 @@
 
 ## operations on menus
 
-proc FillMenu {menu commd descr} {
+proc FillMenu {menu commdargs descr} {
     # entry point for recursive call of FillMenuRec
     #  $menu is the parent menu to fill in
-    #  $commd is the callback to associate to each terminal entry
-    #    whose arguments will be the entry and $menu (even on sub-menus)
+    #  $commdargs is the callback and initial arguments to associate to
+    #    each terminal entry; this is treated as a list to which will be
+    #    appended the entry and $menu (even on sub-menus)
     #  $descr is a list describing the menu contents as follows:
     #    @ LIST  create sub-menu whose label is the head of $LIST, and
     #             whose description is the 2nd and following elements of $LIST
@@ -38,11 +39,11 @@ proc FillMenu {menu commd descr} {
     # sub-menus will be created when menu length would exceed MAXMENUITEMS
 
     $menu delete 0 end
-    FillMenuRec $menu $menu $commd $descr
+    FillMenuRec $menu $menu $commdargs $descr
     return
 }
 
-proc FillMenuRec {w menu commd descr} {
+proc FillMenuRec {w menu commdargs descr} {
     # fill in menus recursively according to description
     # see proc FillMenu for the meaning of the arguments
     global MAXMENUITEMS TXT
@@ -57,10 +58,11 @@ proc FillMenuRec {w menu commd descr} {
 	if { $notsub } {
 	    if { $item != "@" } {
 		if { $item != "---" } {
-		  $menu add command -label $item \
-			  -command [list $commd $item $w]
+		    set cmd $commdargs
+		    lappend cmd $item $w
+		    $menu add command -label $item -command $cmd
 		} else {
-		  $menu add separator
+		    $menu add separator
 		}
 	    } else {
 		set notsub 0 ; incr c -1 ; incr dl -1
@@ -70,7 +72,7 @@ proc FillMenuRec {w menu commd descr} {
 	    set msub $menu.m$c
 	    $menu add cascade -label [lindex $item 0] -menu $msub
 	    destroy $msub ; menu $msub -tearoff 0
-	    FillMenuRec $w $msub $commd [lrange $item 1 end]
+	    FillMenuRec $w $msub $commdargs [lrange $item 1 end]
 	}
 	incr c
     }
@@ -271,7 +273,7 @@ proc PosnGetCheckEmpty {w datum errproc chgvar} {
 		set $m [$w.$m get]
 		if { [set $m] == "" } {
 		    return empty
-		} elseif { ! [CheckNumber $errproc [set $m]] } { return nil }
+		} elseif { ! [CheckFloat $errproc [set $m]] } { return nil }
 	    }
 	    if { $eng != 0 } { set eng [string trimleft $eng "0"] }
 	    if { $nng != 0 } { set nng [string trimleft $nng "0"] }
@@ -715,6 +717,17 @@ proc Delete {l x} {
     return $l
 }
 
+proc Subtract {l1 l2} {
+    # return list obtained from $l1 by deleting all elements in $l2
+
+    foreach x $l2 {
+	if { [set ix [lsearch -exact $l1 $x]] != -1 } {
+	    set l1 [lreplace $l1 $ix $ix]
+	}
+    }
+    return $l1
+}
+
 proc Intersect1 {l1 l2} {
     # return first common element in both lists or empty list if none
 
@@ -739,6 +752,14 @@ proc FindArrayIndices {array val errix} {
     return $errix
 }
 
+proc AssignGlobal {var val} {
+    # assign $val to global $var
+    global $var
+
+    set $var $val
+    return
+}
+
 ## hiding and showing columns of objects in a grid
 
 proc CollapseColumn {objs col label type args} {
@@ -1041,15 +1062,16 @@ proc TurnObject {trk data} {
 
 ## varia
 
-proc FixedFont {size} {
+proc GFixedFont {size} {
     # return fixed font of given size
     # if $size is an integer it is taken to be the size in points, otherwise
-    #  the default of Tcl/Tk for the fixed font is used
+    #  the default for the fixed font is used
+    global FixedFont
 
     if { [regexp {^[0-9]+$} $size] } {
 	return "fixed $size"
     }
-    return fixed
+    return $FixedFont
 }
 
 proc Measure {text} {
